@@ -92,14 +92,6 @@ const short sizeTab[8][4][2] =
 	{{128,128}, {256,256}, {512,256}, {512,512}}, //affine ext direct
 };
 
-static GraphicsInterface_struct *GFXCore=NULL;
-
-// This should eventually be moved to the port specific code
-GraphicsInterface_struct *GFXCoreList[] = {
-&GFXDummy,
-NULL
-};
-
 static const CACHE_ALIGN u8 win_empty[256] = {
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -219,14 +211,14 @@ void GPU_Reset(GPU *g, u8 l)
 
 	if(g->core == GPU_SUB)
 	{
-		g->oam = (OAM *)(MMU.ARM9_OAM + ADDRESS_STEP_1KB);
+		g->oam = (MMU.ARM9_OAM + ADDRESS_STEP_1KB);
 		g->sprMem = MMU_BOBJ;
 		// GPU core B
 		g->dispx_st = (REG_DISPx*)(&MMU.ARM9_REG[REG_DISPB]);
 	}
 	else
 	{
-		g->oam = (OAM *)(MMU.ARM9_OAM);
+		g->oam = (MMU.ARM9_OAM);
 		g->sprMem = MMU_AOBJ;
 		// GPU core A
 		g->dispx_st = (REG_DISPx*)(&MMU.ARM9_REG[0]);
@@ -615,7 +607,7 @@ FORCEINLINE FASTCALL void GPU::_master_setFinal3dColor(int dstX, int srcX)
 			switch(FUNC) {
 				case Increase: final = currentFadeInColors[final&0x7FFF]; break;
 				case Decrease: final = currentFadeOutColors[final&0x7FFF]; break;
-				case None: 
+				case NoBlend: 
 				case Blend:
 					break;
 			}
@@ -630,7 +622,7 @@ template<bool BACKDROP, BlendFunc FUNC, bool WINDOW>
 FORCEINLINE FASTCALL bool GPU::_master_setFinalBGColor(u16 &color, const u32 x)
 {
 	//no further analysis for no special effects. on backdrops. just draw it.
-	if(FUNC==None && BACKDROP) return true;
+	if(FUNC==NoBlend && BACKDROP) return true;
 
 	//blend backdrop with what?? this doesn't make sense
 	if(FUNC==Blend && BACKDROP) return true;
@@ -660,7 +652,7 @@ FORCEINLINE FASTCALL bool GPU::_master_setFinalBGColor(u16 &color, const u32 x)
 		case Blend: if(blend2[bg_under]) color = blend(color,HostReadWord(currDst, x<<1)); break;
 		case Increase: color = currentFadeInColors[color]; break;
 		case Decrease: color = currentFadeOutColors[color]; break;
-		case None: break;
+		case NoBlend: break;
 	}
 	return true;
 }
@@ -693,7 +685,7 @@ static FORCEINLINE void _master_setFinalOBJColor(GPU *gpu, u8 *dst, u16 color, u
 
 		//only when blend color effect is selected, ordinarily opaque sprites are blended with the color effect params
 		case Blend: forceBlendingForNormal = true; break;
-		case None: break;
+		case NoBlend: break;
 		}
 
 	//this inspects the layer beneath the sprite to see if the current blend flags make it a candidate for blending
@@ -730,11 +722,11 @@ FORCEINLINE void GPU::setFinalColorBG(u16 color, const u32 x)
 	const int test = BACKDROP?FUNCNUM:setFinalColorBck_funcNum;
 	switch(test)
 	{
-		case 0: draw = _master_setFinalBGColor<BACKDROP,None,false>(color,x); break;
+		case 0: draw = _master_setFinalBGColor<BACKDROP,NoBlend,false>(color,x); break;
 		case 1: draw = _master_setFinalBGColor<BACKDROP,Blend,false>(color,x); break;
 		case 2: draw = _master_setFinalBGColor<BACKDROP,Increase,false>(color,x); break;
 		case 3: draw = _master_setFinalBGColor<BACKDROP,Decrease,false>(color,x); break;
-		case 4: draw = _master_setFinalBGColor<BACKDROP,None,true>(color,x); break;
+		case 4: draw = _master_setFinalBGColor<BACKDROP,NoBlend,true>(color,x); break;
 		case 5: draw = _master_setFinalBGColor<BACKDROP,Blend,true>(color,x); break;
 		case 6: draw = _master_setFinalBGColor<BACKDROP,Increase,true>(color,x); break;
 		case 7: draw = _master_setFinalBGColor<BACKDROP,Decrease,true>(color,x); break;
@@ -753,11 +745,11 @@ FORCEINLINE void GPU::setFinalColor3d(int dstX, int srcX)
 {
 	switch(setFinalColor3d_funcNum)
 	{
-	case 0x0: _master_setFinal3dColor<None,false>(dstX,srcX); break;
+	case 0x0: _master_setFinal3dColor<NoBlend,false>(dstX,srcX); break;
 	case 0x1: _master_setFinal3dColor<Blend,false>(dstX,srcX); break;
 	case 0x2: _master_setFinal3dColor<Increase,false>(dstX,srcX); break;
 	case 0x3: _master_setFinal3dColor<Decrease,false>(dstX,srcX); break;
-	case 0x4: _master_setFinal3dColor<None,true>(dstX,srcX); break;
+	case 0x4: _master_setFinal3dColor<NoBlend,true>(dstX,srcX); break;
 	case 0x5: _master_setFinal3dColor<Blend,true>(dstX,srcX); break;
 	case 0x6: _master_setFinal3dColor<Increase,true>(dstX,srcX); break;
 	case 0x7: _master_setFinal3dColor<Decrease,true>(dstX,srcX); break;
@@ -768,11 +760,11 @@ FORCEINLINE void setFinalColorSpr(GPU* gpu, u8 *dst, u16 color, u8 alpha, u8 typ
 {
 	switch(gpu->setFinalColorSpr_funcNum)
 	{
-	case 0x0: _master_setFinalOBJColor<None,false>(gpu, dst, color, alpha, type, x); break;
+	case 0x0: _master_setFinalOBJColor<NoBlend,false>(gpu, dst, color, alpha, type, x); break;
 	case 0x1: _master_setFinalOBJColor<Blend,false>(gpu, dst, color, alpha, type, x); break;
 	case 0x2: _master_setFinalOBJColor<Increase,false>(gpu, dst, color, alpha, type, x); break;
 	case 0x3: _master_setFinalOBJColor<Decrease,false>(gpu, dst, color, alpha, type, x); break;
-	case 0x4: _master_setFinalOBJColor<None,true>(gpu, dst, color, alpha, type, x); break;
+	case 0x4: _master_setFinalOBJColor<NoBlend,true>(gpu, dst, color, alpha, type, x); break;
 	case 0x5: _master_setFinalOBJColor<Blend,true>(gpu, dst, color, alpha, type, x); break;
 	case 0x6: _master_setFinalOBJColor<Increase,true>(gpu, dst, color, alpha, type, x); break;
 	case 0x7: _master_setFinalOBJColor<Decrease,true>(gpu, dst, color, alpha, type, x); break;
@@ -822,6 +814,45 @@ finish:
 	}
 }
 
+//unpacks an _OAM_ structure from the provided oam buffer (should point at OAM 0) and provided OAM index.
+//is endian-safe
+void SlurpOAM(_OAM_* oam_output, void* oam_buffer, int oam_index)
+{
+	u16* u16_oam_buffer = (u16*)oam_buffer;
+	int u16_offset = oam_index<<2;
+	u16 attr[4];
+	for(int i=0;i<4;i++)
+		attr[i] = LE_TO_LOCAL_16(u16_oam_buffer[u16_offset + i]);
+	
+	oam_output->Y = (attr[0]>>0) & 0xFF;
+	oam_output->RotScale = (attr[0]>>8)&3;
+	oam_output->Mode = (attr[0]>>10)&3;
+	oam_output->Mosaic = (attr[0]>>12)&1;
+	oam_output->Depth = (attr[0]>>13)&1;
+	oam_output->Shape = (attr[0]>>14)&3;
+	
+	oam_output->X = (((s32)((attr[1]>>0)&0x1FF))<<23)>>23;
+	oam_output->RotScalIndex = (attr[1]>>9)&7;
+	oam_output->HFlip = (attr[1]>>12)&1;
+	oam_output->VFlip = (attr[1]>>13)&1;
+	oam_output->Size = (attr[1]>>14)&3;
+
+	oam_output->TileIndex = (attr[2]>>0)&0x3FF;
+	oam_output->Priority = (attr[2]>>10)&3;
+	oam_output->PaletteIndex = (attr[2]>>12)&0xF;
+
+	oam_output->attr3 = attr[3];
+}
+
+//gets the affine parameter associated with the specified oam index.
+u16 SlurpOAMAffineParam(void* oam_buffer, int oam_index)
+{
+	u16* u16_oam_buffer = (u16*)oam_buffer;
+	int u16_offset = oam_index<<2;
+	return LE_TO_LOCAL_16(u16_oam_buffer[u16_offset + 3]);
+}
+
+
 //this is fantastically inaccurate.
 //we do the early return even though it reduces the resulting accuracy
 //because we need the speed, and because it is inaccurate anyway
@@ -830,8 +861,9 @@ static void mosaicSpriteLinePixel(GPU * gpu, int x, u16 l, u8 * dst, u8 * dst_al
 	int x_int;
 	int y = l;
 
-	_OAM_ * spriteInfo = (_OAM_ *)(gpu->oam + gpu->sprNum[x]);
-	bool enabled = spriteInfo->Mosaic;
+	_OAM_ spriteInfo;
+	SlurpOAM(&spriteInfo,gpu->oam,gpu->sprNum[x]);
+	bool enabled = spriteInfo.Mosaic!=0;
 	if(!enabled)
 		return;
 
@@ -1289,8 +1321,6 @@ template<bool MOSAIC> void lineExtRot(GPU * gpu)
 //			SPRITE RENDERING -HELPER FUNCTIONS-
 /*****************************************************************************/
 
-#define nbShow 128
-
 /* if i understand it correct, and it fixes some sprite problems in chameleon shot */
 /* we have a 15 bit color, and should use the pal entry bits as alpha ?*/
 /* http://nocash.emubase.de/gbatek.htm#dsvideoobjs */
@@ -1481,28 +1511,14 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 	int cost = 0;
 
 	struct _DISPCNT * dispCnt = &(gpu->dispx_st)->dispx_DISPCNT.bits;
-	_OAM_ * spriteInfo = (_OAM_ *)(gpu->oam);
 	u8 block = gpu->sprBoundary;
-	u8 i;
 
-	//what the hell? why is all this here? the #ifdefs in the bitfields definition should take care of this.
-	//this needs to be fixed anyway since i changed the sprite render order
-	//better yet, just dont do it this way at all. _OAM_ is so small, why not just copy it and then twiddle it?
-
-//#ifdef WORDS_BIGENDIAN
-//	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15;
-//	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14;
-//#endif
-
-	for(i = 0; i<nbShow; ++i, ++spriteInfo
-//#ifdef WORDS_BIGENDIAN    
-//	,*(((u16*)(spriteInfo+1))+1) = (*(((u16*)(spriteInfo+1))+1) << 1) | *(((u16*)(spriteInfo+1))+1) >> 15
-//	,*(((u16*)(spriteInfo+1))+2) = (*(((u16*)(spriteInfo+1))+2) << 2) | *(((u16*)(spriteInfo+1))+2) >> 14
-//	,*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15
-//	,*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14
-//#endif
-	)     
+	for(int i = 0; i<128; i++)     
 	{
+		_OAM_ oam;
+		_OAM_* spriteInfo = &oam;
+		SlurpOAM(spriteInfo, gpu->oam, i);
+
 		//for each sprite:
 		if(cost>=2130)
 		{
@@ -1573,17 +1589,11 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 			blockparameter = (spriteInfo->RotScalIndex + (spriteInfo->HFlip<< 3) + (spriteInfo->VFlip << 4))*4;
 
 			// Get rotation/scale parameters
-#ifdef WORDS_BIGENDIAN
-			dx  = ((s16)(gpu->oam + blockparameter+0)->attr31 << 8) | ((s16)(gpu->oam + blockparameter+0)->attr30);
-			dmx = ((s16)(gpu->oam + blockparameter+1)->attr31 << 8) | ((s16)(gpu->oam + blockparameter+1)->attr30);
-			dy  = ((s16)(gpu->oam + blockparameter+2)->attr31 << 8) | ((s16)(gpu->oam + blockparameter+2)->attr30);
-			dmy = ((s16)(gpu->oam + blockparameter+3)->attr31 << 8) | ((s16)(gpu->oam + blockparameter+3)->attr30);
-#else
-			dx  = (s16)(gpu->oam + blockparameter+0)->attr3;
-			dmx = (s16)(gpu->oam + blockparameter+1)->attr3;
-			dy  = (s16)(gpu->oam + blockparameter+2)->attr3;
-			dmy = (s16)(gpu->oam + blockparameter+3)->attr3;
-#endif
+			dx = SlurpOAMAffineParam(gpu->oam,blockparameter+0);
+			dmx = SlurpOAMAffineParam(gpu->oam,blockparameter+1);
+			dy = SlurpOAMAffineParam(gpu->oam,blockparameter+2);
+			dmy = SlurpOAMAffineParam(gpu->oam,blockparameter+3);
+
 
 			// Calculate fixed poitn 8.8 start offsets
 			realX = ((sprSize.x) << 7) - (fieldX >> 1)*dx - (fieldY>>1)*dmx + y * dmx;
@@ -1832,11 +1842,6 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 		}
 	}
 
-//#ifdef WORDS_BIGENDIAN
-//	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) << 1) | *(((u16*)spriteInfo)+1) >> 15;
-//	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) << 2) | *(((u16*)spriteInfo)+2) >> 14;
-//#endif
-
 }
 
 
@@ -1844,7 +1849,7 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 //			SCREEN FUNCTIONS
 /*****************************************************************************/
 
-int Screen_Init(int coreid)
+int Screen_Init()
 {
 	MainScreen.gpu = GPU_Init(0);
 	SubScreen.gpu = GPU_Init(1);
@@ -1857,7 +1862,7 @@ int Screen_Init(int coreid)
 	if (osd)  {delete osd; osd =NULL; }
 	osd  = new OSDCLASS(-1);
 
-	return GPU_ChangeGraphicsCore(coreid);
+	return 0;
 }
 
 void Screen_Reset(void)
@@ -1878,89 +1883,9 @@ void Screen_DeInit(void)
 	GPU_DeInit(MainScreen.gpu);
 	GPU_DeInit(SubScreen.gpu);
 
-	if (GFXCore)
-		GFXCore->DeInit();
-
 	if (osd)  {delete osd; osd =NULL; }
 }
 
-/*****************************************************************************/
-//			GRAPHICS CORE
-/*****************************************************************************/
-
-// This is for future graphics core switching. This is by no means set in stone
-
-int GPU_ChangeGraphicsCore(int coreid)
-{
-   int i;
-
-   // Make sure the old core is freed
-   if (GFXCore)
-      GFXCore->DeInit();
-
-   // So which core do we want?
-   if (coreid == GFXCORE_DEFAULT)
-      coreid = 0; // Assume we want the first one
-
-   // Go through core list and find the id
-   for (i = 0; GFXCoreList[i] != NULL; i++)
-   {
-      if (GFXCoreList[i]->id == coreid)
-      {
-         // Set to current core
-         GFXCore = GFXCoreList[i];
-         break;
-      }
-   }
-
-   if (GFXCore == NULL)
-   {
-      GFXCore = &GFXDummy;
-      return -1;
-   }
-
-   if (GFXCore->Init() == -1)
-   {
-      // Since it failed, instead of it being fatal, we'll just use the dummy
-      // core instead
-      GFXCore = &GFXDummy;
-   }
-
-   return 0;
-}
-
-int GFXDummyInit();
-void GFXDummyDeInit();
-void GFXDummyResize(int width, int height, BOOL fullscreen);
-void GFXDummyOnScreenText(char *string, ...);
-
-GraphicsInterface_struct GFXDummy = {
-GFXCORE_DUMMY,
-"Dummy Graphics Interface",
-0,
-GFXDummyInit,
-GFXDummyDeInit,
-GFXDummyResize,
-GFXDummyOnScreenText
-};
-
-int GFXDummyInit()
-{
-   return 0;
-}
-
-void GFXDummyDeInit()
-{
-}
-
-void GFXDummyResize(int width, int height, BOOL fullscreen)
-{
-}
-
-void GFXDummyOnScreenText(char *string, ...)
-{
-}
-
 
 /*****************************************************************************/
 //			GPU_RenderLine
@@ -2038,8 +1963,9 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 	switch(gpu->setFinalColorBck_funcNum)
 	{
 		//for backdrops, blend isnt applied (it's illogical, isnt it?)
-		case 0: 
-		case 1: 
+		case 0:
+		case 1:
+PLAIN_CLEAR:
 			memset_u16_le<256>(gpu->currDst,backdrop_color); 
 			break;
 
@@ -2047,10 +1973,12 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 		case 2:
 			if(gpu->BLDCNT & 0x20) //backdrop is selected for color effect
 				memset_u16_le<256>(gpu->currDst,gpu->currentFadeInColors[backdrop_color]);
+			else goto PLAIN_CLEAR;
 			break;
 		case 3:
 			if(gpu->BLDCNT & 0x20) //backdrop is selected for color effect
 				memset_u16_le<256>(gpu->currDst,gpu->currentFadeOutColors[backdrop_color]);
+			else goto PLAIN_CLEAR;
 			break;
 
 		//windowed cases apparently need special treatment? why? can we not render the backdrop? how would that even work?
