@@ -32,6 +32,10 @@
 #include "lua-engine.h"
 #endif
 
+#ifdef HAVE_JIT
+#include "arm_jit.h"
+#endif
+
 #define ARMCPU_ARM7 1
 #define ARMCPU_ARM9 0
 #define ARMPROC (PROCNUM ? NDS_ARM7:NDS_ARM9)
@@ -313,6 +317,11 @@ typedef struct
 	
 } nds_dscard;
 
+#define DUP2(x)  x, x
+#define DUP4(x)  x, x, x, x
+#define DUP8(x)  x, x, x, x,  x, x, x, x
+#define DUP16(x) x, x, x, x,  x, x, x, x,  x, x, x, x,  x, x, x, x
+
 struct MMU_struct 
 {
 	//ARM9 mem
@@ -348,16 +357,16 @@ struct MMU_struct
 
 	//ARM7 mem
 	u8 ARM7_BIOS[0x4000];
-	u8 ARM7_ERAM[0x10000];
+	u8 ARM7_ERAM[0x10000]; //64KB of exclusive WRAM
 	u8 ARM7_REG[0x10000];
-	u8 ARM7_WIRAM[0x10000];
+	u8 ARM7_WIRAM[0x10000]; //WIFI ram
 
 	// VRAM mapping
 	u8 VRAM_MAP[4][32];
 	u32 LCD_VRAM_ADDR[10];
 	u8 LCDCenable[10];
 
-	//Shared ram
+	//32KB of shared WRAM - can be switched between ARM7 & ARM9 in two blocks
 	u8 SWIRAM[0x8000];
 
 	//Card rom & ram
@@ -411,6 +420,8 @@ struct MMU_struct
 	u16 AUX_SPI_CNT;
 	u16 AUX_SPI_CMD;
 
+	u8 WRAMCNT;
+
 	u64 gfx3dCycles;
 
 	u8 powerMan_CntReg;
@@ -748,10 +759,6 @@ FORCEINLINE u32 _MMU_read32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u
 	{
 		if ( (addr & 0x0F000000) == 0x02000000)
 			return T1ReadLong_guaranteedAligned( MMU.MAIN_MEM, addr & _MMU_MAIN_MEM_MASK32);
-		else if((addr & 0xFF800000) == 0x03800000)
-			return T1ReadLong_guaranteedAligned(MMU.ARM7_ERAM, addr&0xFFFC);
-		else if((addr & 0xFF800000) == 0x03000000)
-			return T1ReadLong_guaranteedAligned(MMU.SWIRAM, addr&0x7FFC);
 	}
 
 
@@ -795,6 +802,9 @@ FORCEINLINE void _MMU_write08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 		}
 
 	if ( (addr & 0x0F000000) == 0x02000000) {
+#ifdef HAVE_JIT
+		JIT_COMPILED_FUNC_KNOWNBANK(addr, MAIN_MEM, _MMU_MAIN_MEM_MASK, 0) = 0;
+#endif
 		T1WriteByte( MMU.MAIN_MEM, addr & _MMU_MAIN_MEM_MASK, val);
 #ifdef HAVE_LUA
 		CallRegisteredLuaMemHook(addr, 1, val, LUAMEMHOOK_WRITE);
@@ -831,6 +841,9 @@ FORCEINLINE void _MMU_write16(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 		}
 
 	if ( (addr & 0x0F000000) == 0x02000000) {
+#ifdef HAVE_JIT
+		JIT_COMPILED_FUNC_KNOWNBANK(addr, MAIN_MEM, _MMU_MAIN_MEM_MASK16, 0) = 0;
+#endif
 		T1WriteWord( MMU.MAIN_MEM, addr & _MMU_MAIN_MEM_MASK16, val);
 #ifdef HAVE_LUA
 		CallRegisteredLuaMemHook(addr, 2, val, LUAMEMHOOK_WRITE);
@@ -867,6 +880,10 @@ FORCEINLINE void _MMU_write32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 		}
 
 	if ( (addr & 0x0F000000) == 0x02000000) {
+#ifdef HAVE_JIT
+		JIT_COMPILED_FUNC_KNOWNBANK(addr, MAIN_MEM, _MMU_MAIN_MEM_MASK32, 0) = 0;
+		JIT_COMPILED_FUNC_KNOWNBANK(addr, MAIN_MEM, _MMU_MAIN_MEM_MASK32, 1) = 0;
+#endif
 		T1WriteLong( MMU.MAIN_MEM, addr & _MMU_MAIN_MEM_MASK32, val);
 #ifdef HAVE_LUA
 		CallRegisteredLuaMemHook(addr, 4, val, LUAMEMHOOK_WRITE);
