@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2012 DeSmuME team
+	Copyright (C) 2008-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -177,6 +177,7 @@ struct NDSSystem
 	u16 adc_touchX;
 	u16 adc_touchY;
 	s32 adc_jitterctr;
+	BOOL stylusJitter;
 
 	//the DSI returns calibrated touch coords from its TSC (?), so we need to save these separately
 	u16 scr_touchX;
@@ -442,7 +443,7 @@ void NDS_suspendProcessingInput(bool suspend);
 
 
 
-int NDS_LoadROM(const char *filename, const char* logicalFilename=0);
+int NDS_LoadROM(const char *filename, const char* physicalFilename=0, const char* logicalFilename=0);
 void NDS_FreeROM(void);
 void NDS_Reset();
 int NDS_ImportSave(const char *filename, u32 force_size = 0);
@@ -455,7 +456,7 @@ bool nds_loadstate(EMUFILE* is, int size);
 int NDS_WriteBMP(const char *filename);
 
 void NDS_Sleep();
-void NDS_ToggleCardEject();
+void NDS_TriggerCardEjectIRQ();
 
 void NDS_SkipNextFrame();
 #define NDS_SkipFrame(s) if(s) NDS_SkipNext2DFrame();
@@ -488,6 +489,8 @@ extern struct TCommonSettings {
 		, GFX3D_Texture(true)
 		, GFX3D_LineHack(true)
 		, GFX3D_Zelda_Shadow_Depth_Hack(0)
+		, GFX3D_Renderer_Multisample(false)
+		, jit_max_block_size(100)
 		, UseExtBIOS(false)
 		, SWIFromBIOS(false)
 		, PatchSWI3(false)
@@ -507,6 +510,7 @@ extern struct TCommonSettings {
 		, spu_advanced(false)
 		, StylusPressure(50)
 		, ConsoleType(NDS_CONSOLE_TYPE_FAT)
+		, StylusJitter(false)
 	{
 		strcpy(ARM9BIOS, "biosnds9.bin");
 		strcpy(ARM7BIOS, "biosnds7.bin");
@@ -523,6 +527,13 @@ extern struct TCommonSettings {
 		for(int g=0;g<2;g++)
 			for(int x=0;x<5;x++)
 				dispLayers[g][x]=true;
+#ifdef HAVE_JIT
+		//zero 06-sep-2012 - shouldnt be defaulting this to true for now, since the jit is buggy. 
+		//id rather have people discover a bonus speedhack than discover new bugs in a new version
+		use_jit = false;
+#else
+		use_jit = false;
+#endif
 	}
 	bool GFX3D_HighResolutionInterpolateColor;
 	bool GFX3D_EdgeMark;
@@ -530,6 +541,7 @@ extern struct TCommonSettings {
 	bool GFX3D_Texture;
 	bool GFX3D_LineHack;
 	int  GFX3D_Zelda_Shadow_Depth_Hack;
+	bool GFX3D_Renderer_Multisample;
 
 	bool UseExtBIOS;
 	char ARM9BIOS[256];
@@ -553,10 +565,14 @@ extern struct TCommonSettings {
 	bool rigorous_timing;
 
 	int StylusPressure;
+	bool StylusJitter;
 
 	bool dispLayers[2][5];
 	
 	FAST_ALIGN bool advanced_timing;
+
+	bool use_jit;
+	u32	jit_max_block_size;
 	
 	struct _Wifi {
 		int mode;
