@@ -25,6 +25,7 @@
 #include "types.h"
 #include "matrix.h"
 #include "emufile.h"
+#include "metaspu/metaspu.h"
 
 
 #define SNDCORE_DEFAULT         -1
@@ -60,6 +61,8 @@ struct SoundInterface_struct
    void (*UnMuteAudio)();
    void (*SetVolume)(int volume);
    void (*ClearBuffer)();
+	void (*FetchSamples)(s16 *sampleBuffer, size_t sampleCount, ESynchMode synchMode, ISynchronizingAudioBuffer *theSynchronizer);
+	size_t (*PostProcessSamples)(s16 *postProcessBuffer, size_t requestedSampleCount, ESynchMode synchMode, ISynchronizingAudioBuffer *theSynchronizer);
 };
 
 extern SoundInterface_struct SNDDummy;
@@ -86,10 +89,6 @@ struct channel_struct
    u32 length;
    u32 totlength;
    double double_totlength_shifted;
-   union {
-		s8 *buf8;
-		s16 *buf16;
-   };
    double sampcnt;
    double sampinc;
    // ADPCM specific
@@ -212,6 +211,8 @@ u16 SPU_ReadWord(u32 addr);
 u32 SPU_ReadLong(u32 addr);
 void SPU_Emulate_core(void);
 void SPU_Emulate_user(bool mix = true);
+void SPU_DefaultFetchSamples(s16 *sampleBuffer, size_t sampleCount, ESynchMode synchMode, ISynchronizingAudioBuffer *theSynchronizer);
+size_t SPU_DefaultPostProcessSamples(s16 *postProcessBuffer, size_t requestedSampleCount, ESynchMode synchMode, ISynchronizingAudioBuffer *theSynchronizer);
 
 extern SPU_struct *SPU_core, *SPU_user;
 extern int spu_core_samples;
