@@ -1,20 +1,18 @@
-/*  Copyright (C) 2009-2010 DeSmuME team
+/*
+	Copyright (C) 2009-2011 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef __ADDONS_H__
@@ -42,14 +40,14 @@ struct ADDONINTERFACE
 	void (*config)(void);
 
 	//called when the emulator write to addon
-	void (*write08)(u32 adr, u8 val);
-	void (*write16)(u32 adr, u16 val);
-	void (*write32)(u32 adr, u32 val);
+	void (*write08)(u32 procnum, u32 adr, u8 val);
+	void (*write16)(u32 procnum, u32 adr, u16 val);
+	void (*write32)(u32 procnum, u32 adr, u32 val);
 
 	//called when the emulator read from addon
-	u8  (*read08)(u32 adr);
-	u16 (*read16)(u32 adr);
-	u32 (*read32)(u32 adr);
+	u8  (*read08)(u32 procnum, u32 adr);
+	u16 (*read16)(u32 procnum, u32 adr);
+	u32 (*read32)(u32 procnum, u32 adr);
 	
 	//called when the user get info about addon pak (description)
 	void (*info)(char *info);
@@ -64,6 +62,7 @@ enum NDS_ADDON_TYPE
 	NDS_ADDON_GUITARGRIP,	// Guitar Grip
 	NDS_ADDON_EXPMEMORY,	// Memory Expansion 
 	NDS_ADDON_PIANO,		// Piano
+  NDS_ADDON_PADDLE,
 	//NDS_ADDON_EXTERNALMIC,
 	NDS_ADDON_COUNT		// use for counter addons - MUST TO BE LAST!!!
 };
