@@ -1,30 +1,31 @@
-/*  Copyright (C) 2009-2010 DeSmuME team
+/*
+	Copyright (C) 2009-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <stdlib.h>
 #include <string.h>
 #include "common.h"
 #include <vector>
 
-#define CHEAT_VERSION_MAJOR		2
-#define CHEAT_VERSION_MINOR		0
-#define MAX_CHEAT_LIST			100
-#define	MAX_XX_CODE				255
+#define CHEAT_VERSION_MAJOR			2
+#define CHEAT_VERSION_MINOR			0
+#define MAX_CHEAT_LIST				100
+#define	MAX_XX_CODE					1024
+#define CHEAT_FILE_MIN_FGETS_BUFFER	32768
+#define CHEAT_DB_GAME_TITLE_SIZE	256
 
 struct CHEATS_LIST
 {
@@ -37,8 +38,12 @@ struct CHEATS_LIST
 								// 1 - Action Replay
 								// 2 - Codebreakers
 	BOOL	enabled;
+	// TODO
+	u8		freezeType;			// 0 - normal freeze
+								// 1 - can decrease
+								// 2 - can increase
 	u32		code[MAX_XX_CODE][2];
-	char	description[255];
+	char	description[1024];
 	int		num;
 	u8		size;
 };
@@ -52,7 +57,6 @@ private:
 
 	void	clear();
 	void	ARparser(CHEATS_LIST& cheat);
-	BOOL	XXcodePreParser(CHEATS_LIST *cheat, char *code);
 	char	*clearCode(char *s);
 
 public:
@@ -74,12 +78,18 @@ public:
 	BOOL	remove(u32 pos);
 	void	getListReset();
 	BOOL	getList(CHEATS_LIST *cheat);
+	CHEATS_LIST*	getListPtr();
 	BOOL	get(CHEATS_LIST *cheat, u32 pos);
+	CHEATS_LIST*	getItemByIndex(const u32 pos);
 	u32		getSize();
+	void	setDescription(const char *description, u32 pos);
 	BOOL	save();
 	BOOL	load();
 	void	process();
 	void	getXXcodeString(CHEATS_LIST cheat, char *res_buf);
+	
+	static BOOL XXCodeFromString(CHEATS_LIST *cheatItem, const std::string codeString);
+	static BOOL XXCodeFromString(CHEATS_LIST *cheatItem, const char *codeString);
 };
 
 class CHEATSEARCH
@@ -115,44 +125,68 @@ enum CHEATS_DB_TYPE
 
 #pragma pack(push)
 #pragma pack(1)
-	typedef struct FAT_R4
-	{
-		u8	serial[4];
-		u32	CRC;
-		u64 addr;
-	} FAT_R4;
+typedef struct FAT_R4
+{
+	u8	serial[4];
+	u32	CRC;
+	u64 addr;
+} FAT_R4;
 #pragma pack(pop)
 
 class CHEATSEXPORT
 {
 private:
 	CHEATS_DB_TYPE		type;
+	bool				encrypted;
 	FILE				*fp;
 	u32					fsize;
 	u32					dataSize;
+	u32					encOffset;
 	FAT_R4				fat;
 	bool				search();
 	bool				getCodes();
+	void				R4decrypt(u8 *buf, u32 len, u32 n);
 
 	u32					numCheats;
 	CHEATS_LIST			*cheats;
 
+	u8					error;		//	0 - no errors
+									//	1 - open failed/file not found
+									//	2 - file format is wrong (no valid header ID)
+									//	3 - cheat not found in database
+									//	4 - export error from database
+
 public:
 	CHEATSEXPORT() :
 			fp(NULL),
 			fsize(0),
 			dataSize(0),
+			encOffset(0),
 			type(CHEATS_DB_R4),
+			encrypted(false),
 			numCheats(0),
-			cheats(0)
-	{}
+			cheats(0),
+			CRC(0),
+			error(0)
+	{
+		memset(date, 0, sizeof(date));
+		gametitle = (u8 *)malloc(CHEAT_DB_GAME_TITLE_SIZE);
+		memset(gametitle, 0, CHEAT_DB_GAME_TITLE_SIZE);
+	}
 	~CHEATSEXPORT()
-	{}
+	{
+		free(gametitle);
+		gametitle = NULL;
+	}
 
+	u8				*gametitle;
+	u8				date[17];
+	u32				CRC;
 	bool			load(char *path);
 	void			close();
 	CHEATS_LIST		*getCheats();
 	u32				getCheatsNum();
+	u8				getErrorCode() { return error; }
 };
 
 extern CHEATS *cheats;
