@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2011-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -19,15 +19,18 @@
 #import "cocoa_core.h"
 #import "cocoa_input.h"
 #import "cocoa_firmware.h"
+#import "cocoa_GPU.h"
 #import "cocoa_globals.h"
 #import "cocoa_output.h"
 #import "cocoa_rom.h"
 #import "cocoa_util.h"
+
 #include <mach/mach.h>
 #include <mach/mach_time.h>
 
 #include "../addons.h"
 #include "../NDSSystem.h"
+#include "../slot1.h"
 #undef BOOL
 
 
@@ -38,6 +41,7 @@ volatile bool execute = true;
 
 @dynamic cdsController;
 @synthesize cdsFirmware;
+@synthesize cdsGPU;
 @synthesize cdsOutputList;
 
 @dynamic masterExecute;
@@ -49,6 +53,7 @@ volatile bool execute = true;
 
 @dynamic emulationFlags;
 @synthesize emuFlagAdvancedBusLevelTiming;
+@synthesize emuFlagRigorousTiming;
 @synthesize emuFlagUseExternalBios;
 @synthesize emuFlagEmulateBiosInterrupts;
 @synthesize emuFlagPatchDelayLoop;
@@ -56,12 +61,17 @@ volatile bool execute = true;
 @synthesize emuFlagFirmwareBoot;
 @synthesize emuFlagDebugConsole;
 @synthesize emuFlagEmulateEnsata;
+@dynamic cpuEmulationEngine;
+@dynamic maxJITBlockSize;
+@synthesize slot1DeviceType;
+@synthesize slot1StatusText;
 
 @dynamic arm9ImageURL;
 @dynamic arm7ImageURL;
 @dynamic firmwareImageURL;
+@synthesize slot1R4URL;
 
-@synthesize mutexCoreExecute;
+@dynamic mutexCoreExecute;
 
 static BOOL isCoreStarted = NO;
 
@@ -75,10 +85,12 @@ static BOOL isCoreStarted = NO;
 	
 	cdsController = nil;
 	cdsFirmware = nil;
+	cdsGPU = [[[[CocoaDSGPU alloc] init] autorelease] retain];
 	cdsOutputList = [[NSMutableArray alloc] initWithCapacity:32];
 	
 	emulationFlags = EMULATION_ADVANCED_BUS_LEVEL_TIMING_MASK;
 	emuFlagAdvancedBusLevelTiming = YES;
+	emuFlagRigorousTiming = NO;
 	emuFlagUseExternalBios = NO;
 	emuFlagEmulateBiosInterrupts = NO;
 	emuFlagPatchDelayLoop = NO;
@@ -87,18 +99,23 @@ static BOOL isCoreStarted = NO;
 	emuFlagDebugConsole = NO;
 	emuFlagEmulateEnsata = NO;
 	
-	mutexCoreExecute = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
-	pthread_mutex_init(mutexCoreExecute, NULL);
+	slot1DeviceType = NDS_SLOT1_RETAIL;
+	slot1StatusText = NSSTRING_STATUS_EMULATION_NOT_RUNNING;
+	
 	spinlockMasterExecute = OS_SPINLOCK_INIT;
 	spinlockCdsController = OS_SPINLOCK_INIT;
 	spinlockExecutionChange = OS_SPINLOCK_INIT;
 	spinlockCheatEnableFlag = OS_SPINLOCK_INIT;
 	spinlockEmulationFlags = OS_SPINLOCK_INIT;
+	spinlockCPUEmulationEngine = OS_SPINLOCK_INIT;
 	
 	isSpeedLimitEnabled = YES;
 	speedScalar = SPEED_SCALAR_NORMAL;
 	prevCoreState = CORESTATE_PAUSE;
 	
+	slot1R4URL = nil;
+	_slot1R4Path = "";
+	
 	threadParam.cdsCore = self;
 	threadParam.cdsController = cdsController;
 	threadParam.state = CORESTATE_PAUSE;
@@ -111,11 +128,14 @@ static BOOL isCoreStarted = NO;
 	threadParam.timeBudgetMachAbsTime = *(uint64_t *)&timeBudgetAbsTime;
 	
 	threadParam.exitThread = false;
-	threadParam.mutexCoreExecute = mutexCoreExecute;
+	pthread_mutex_init(&threadParam.mutexCoreExecute, NULL);
+	pthread_mutex_init(&threadParam.mutexOutputList, NULL);
 	pthread_mutex_init(&threadParam.mutexThreadExecute, NULL);
 	pthread_cond_init(&threadParam.condThreadExecute, NULL);
 	pthread_create(&coreThread, NULL, &RunCoreThread, &threadParam);
 	
+	[cdsGPU setMutexProducer:self.mutexCoreExecute];
+	
 	return self;
 }
 
@@ -145,15 +165,16 @@ static BOOL isCoreStarted = NO;
 	pthread_mutex_destroy(&threadParam.mutexThreadExecute);
 	pthread_cond_destroy(&threadParam.condThreadExecute);
 	
-	pthread_mutex_destroy(self.mutexCoreExecute);
-	free(self.mutexCoreExecute);
-	mutexCoreExecute = nil;
-	
 	self.cdsController = nil;
 	self.cdsFirmware = nil;
+	self.cdsGPU = nil;
+	
 	[self removeAllOutputs];
 	[cdsOutputList release];
 	
+	pthread_mutex_destroy(&threadParam.mutexOutputList);
+	pthread_mutex_destroy(&threadParam.mutexCoreExecute);
+	
 	[super dealloc];
 }
 
@@ -197,30 +218,14 @@ static BOOL isCoreStarted = NO;
 - (void) setMasterExecute:(BOOL)theState
 {
 	OSSpinLockLock(&spinlockMasterExecute);
-	
-	if (theState)
-	{
-		execute = true;
-	}
-	else
-	{
-		execute = false;
-	}
-	
+	execute = theState ? true : false;
 	OSSpinLockUnlock(&spinlockMasterExecute);
 }
 
 - (BOOL) masterExecute
 {
-	BOOL theState = NO;
-	
 	OSSpinLockLock(&spinlockMasterExecute);
-	
-	if (execute)
-	{
-		theState = YES;
-	}
-		
+	const BOOL theState = execute ? YES : NO;
 	OSSpinLockUnlock(&spinlockMasterExecute);
 	
 	return theState;
@@ -278,17 +283,10 @@ static BOOL isCoreStarted = NO;
 
 - (BOOL) isFrameSkipEnabled
 {
-	BOOL theState = NO;
-	
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
-	bool cState = threadParam.isFrameSkipEnabled;
+	const BOOL theState = threadParam.isFrameSkipEnabled ? YES : NO;
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	
-	if (cState)
-	{
-		theState = YES;
-	}
-	
 	return theState;
 }
 
@@ -303,7 +301,7 @@ static BOOL isCoreStarted = NO;
 - (CGFloat) speedScalar
 {
 	OSSpinLockLock(&spinlockExecutionChange);
-	CGFloat scalar = speedScalar;
+	const CGFloat scalar = speedScalar;
 	OSSpinLockUnlock(&spinlockExecutionChange);
 	
 	return scalar;
@@ -320,7 +318,7 @@ static BOOL isCoreStarted = NO;
 - (BOOL) isSpeedLimitEnabled
 {
 	OSSpinLockLock(&spinlockExecutionChange);
-	BOOL enabled = isSpeedLimitEnabled;
+	const BOOL enabled = isSpeedLimitEnabled;
 	OSSpinLockUnlock(&spinlockExecutionChange);
 	
 	return enabled;
@@ -329,30 +327,14 @@ static BOOL isCoreStarted = NO;
 - (void) setIsCheatingEnabled:(BOOL)theState
 {
 	OSSpinLockLock(&spinlockCheatEnableFlag);
-	
-	if (theState)
-	{
-		CommonSettings.cheatsDisable = false;
-	}
-	else
-	{
-		CommonSettings.cheatsDisable = true;
-	}
-	
+	CommonSettings.cheatsDisable = theState ? false : true;
 	OSSpinLockUnlock(&spinlockCheatEnableFlag);
 }
 
 - (BOOL) isCheatingEnabled
 {
-	BOOL theState = YES;
-	
 	OSSpinLockLock(&spinlockCheatEnableFlag);
-	
-	if (CommonSettings.cheatsDisable)
-	{
-		theState = NO;
-	}
-	
+	BOOL theState = CommonSettings.cheatsDisable ? NO : YES;
 	OSSpinLockUnlock(&spinlockCheatEnableFlag);
 	
 	return theState;
@@ -364,7 +346,7 @@ static BOOL isCoreStarted = NO;
 	emulationFlags = theFlags;
 	OSSpinLockUnlock(&spinlockEmulationFlags);
 	
-	pthread_mutex_lock(self.mutexCoreExecute);
+	pthread_mutex_lock(&threadParam.mutexCoreExecute);
 	
 	if (theFlags & EMULATION_ADVANCED_BUS_LEVEL_TIMING_MASK)
 	{
@@ -377,6 +359,17 @@ static BOOL isCoreStarted = NO;
 		CommonSettings.advanced_timing = false;
 	}
 	
+	if (theFlags & EMULATION_RIGOROUS_TIMING_MASK)
+	{
+		self.emuFlagRigorousTiming = YES;
+		CommonSettings.rigorous_timing = true;
+	}
+	else
+	{
+		self.emuFlagRigorousTiming = NO;
+		CommonSettings.rigorous_timing = false;
+	}
+	
 	if (theFlags & EMULATION_ENSATA_MASK)
 	{
 		self.emuFlagEmulateEnsata = YES;
@@ -454,18 +447,54 @@ static BOOL isCoreStarted = NO;
 		CommonSettings.DebugConsole = false;
 	}
 	
-	pthread_mutex_unlock(self.mutexCoreExecute);
+	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
 }
 
 - (NSUInteger) emulationFlags
 {
 	OSSpinLockLock(&spinlockEmulationFlags);
-	NSUInteger theFlags = emulationFlags;
+	const NSUInteger theFlags = emulationFlags;
 	OSSpinLockUnlock(&spinlockEmulationFlags);
 	
 	return theFlags;
 }
 
+- (void) setCpuEmulationEngine:(NSInteger)engineID
+{
+	OSSpinLockLock(&spinlockCPUEmulationEngine);
+#if defined(__i386__) || defined(__x86_64__)
+	cpuEmulationEngine = engineID;
+#else
+	cpuEmulationEngine = CPU_EMULATION_ENGINE_INTERPRETER;
+#endif
+	OSSpinLockUnlock(&spinlockCPUEmulationEngine);
+}
+
+- (NSInteger) cpuEmulationEngine
+{
+	OSSpinLockLock(&spinlockCPUEmulationEngine);
+	const NSInteger engineID = cpuEmulationEngine;
+	OSSpinLockUnlock(&spinlockCPUEmulationEngine);
+	
+	return engineID;
+}
+
+- (void) setMaxJITBlockSize:(NSInteger)blockSize
+{
+	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	CommonSettings.jit_max_block_size = (blockSize > 0) ? blockSize : 1;
+	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+}
+
+- (NSInteger) maxJITBlockSize
+{
+	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	const NSInteger blockSize = CommonSettings.jit_max_block_size;
+	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	
+	return blockSize;
+}
+
 - (void) setCoreState:(NSInteger)coreState
 {
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
@@ -487,7 +516,7 @@ static BOOL isCoreStarted = NO;
 - (NSInteger) coreState
 {
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
-	NSInteger theState = threadParam.state;
+	const NSInteger theState = threadParam.state;
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	
 	return theState;
@@ -544,6 +573,11 @@ static BOOL isCoreStarted = NO;
 	return [NSURL fileURLWithPath:[NSString stringWithCString:CommonSettings.Firmware encoding:NSUTF8StringEncoding]];
 }
 
+- (pthread_mutex_t *) mutexCoreExecute
+{
+	return &threadParam.mutexCoreExecute;
+}
+
 - (void) setEjectCardFlag
 {
 	if (nds.cardEjected)
@@ -566,17 +600,20 @@ static BOOL isCoreStarted = NO;
 	return NO;
 }
 
-- (void) toggleEjectCard
+- (void) slot1Eject
 {
-	NDS_ToggleCardEject();
-	[self setEjectCardFlag];
+	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	NDS_TriggerCardEjectIRQ();
+	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	
+	[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_NO_DEVICE];
 }
 
 - (void) changeRomSaveType:(NSInteger)saveTypeID
 {
-	pthread_mutex_lock(self.mutexCoreExecute);
+	pthread_mutex_lock(&threadParam.mutexCoreExecute);
 	[CocoaDSRom changeRomSaveType:saveTypeID];
-	pthread_mutex_unlock(self.mutexCoreExecute);
+	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
 }
 
 - (void) changeExecutionSpeed
@@ -589,7 +626,7 @@ static BOOL isCoreStarted = NO;
 			theSpeed = SPEED_SCALAR_MIN;
 		}
 		
-		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		pthread_mutex_lock(&threadParam.mutexThreadExecute);
 		uint64_t timeBudgetNanoseconds = (uint64_t)(DS_SECONDS_PER_FRAME * 1000000000.0 / theSpeed);
 		AbsoluteTime timeBudgetAbsTime = NanosecondsToAbsolute(*(Nanoseconds *)&timeBudgetNanoseconds);
 		threadParam.timeBudgetMachAbsTime = *(uint64_t *)&timeBudgetAbsTime;
@@ -597,12 +634,79 @@ static BOOL isCoreStarted = NO;
 	}
 	else
 	{
-		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		pthread_mutex_lock(&threadParam.mutexThreadExecute);
 		threadParam.timeBudgetMachAbsTime = 0;
 		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	}
 }
 
+/********************************************************************************************
+	applyDynaRec
+
+	Sets the use_jit variable for CommonSettings.
+
+	Takes:
+		Nothing.
+
+	Returns:
+		Nothing.
+
+	Details:
+		In the UI, we call setCpuEmulationEngine to set whether we should use the
+		interpreter or the dynamic recompiler. However, the emulator cannot handle
+		changing the engine while the emulation is running. Therefore, we use this
+		method to set the engine at a later time, using the last cpuEmulationEngine
+		value from the user.
+ ********************************************************************************************/
+- (void) applyDynaRec
+{
+	const NSInteger engineID = [self cpuEmulationEngine];
+	
+	pthread_mutex_lock(&threadParam.mutexThreadExecute);
+	CommonSettings.use_jit = (engineID == CPU_EMULATION_ENGINE_DYNAMIC_RECOMPILER);
+	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+}
+
+- (BOOL) applySlot1Device
+{
+	const NSInteger deviceTypeID = [self slot1DeviceType];
+	NSString *r4Path = [[self slot1R4URL] path];
+	
+	pthread_mutex_lock(&threadParam.mutexThreadExecute);
+	
+	_slot1R4Path = (r4Path != nil) ? std::string([r4Path cStringUsingEncoding:NSUTF8StringEncoding]) : "";
+	slot1SetFatDir(_slot1R4Path);
+	
+	BOOL result = slot1Change((NDS_SLOT1_TYPE)deviceTypeID);
+	
+	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+	
+	switch (deviceTypeID)
+	{
+		case NDS_SLOT1_NONE:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_NO_DEVICE];
+			break;
+			
+		case NDS_SLOT1_RETAIL:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_RETAIL_INSERTED];
+			break;
+			
+		case NDS_SLOT1_RETAIL_NAND:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_RETAIL_NAND_INSERTED];
+			break;
+			
+		case NDS_SLOT1_R4:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_R4_INSERTED];
+			break;
+			
+		default:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_UNKNOWN_STATE];
+			break;
+	}
+	
+	return result;
+}
+
 - (void) restoreCoreState
 {
 	[self setCoreState:prevCoreState];
@@ -611,6 +715,8 @@ static BOOL isCoreStarted = NO;
 - (void) reset
 {
 	[self setCoreState:CORESTATE_PAUSE];
+	[self applyDynaRec];
+	[self applySlot1Device];
 	
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
 	NDS_Reset();
@@ -622,18 +728,24 @@ static BOOL isCoreStarted = NO;
 
 - (void) addOutput:(CocoaDSOutput *)theOutput
 {
+	pthread_mutex_lock(&threadParam.mutexOutputList);
 	theOutput.mutexProducer = self.mutexCoreExecute;
 	[self.cdsOutputList addObject:theOutput];
+	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
 - (void) removeOutput:(CocoaDSOutput *)theOutput
 {
+	pthread_mutex_lock(&threadParam.mutexOutputList);
 	[self.cdsOutputList removeObject:theOutput];
+	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
 - (void) removeAllOutputs
 {
+	pthread_mutex_lock(&threadParam.mutexOutputList);
 	[self.cdsOutputList removeAllObjects];
+	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
 - (void) runThread:(id)object
@@ -674,7 +786,7 @@ static void* RunCoreThread(void *arg)
 		
 		// Get the user's input, execute a single emulation frame, and generate
 		// the frame output.
-		[(CocoaDSController *)param->cdsController update];
+		[(CocoaDSController *)param->cdsController flush];
 		
 		NDS_beginProcessingInput();
 		// Shouldn't need to do any special processing steps in between.
@@ -682,9 +794,9 @@ static void* RunCoreThread(void *arg)
 		NDS_endProcessingInput();
 		
 		// Execute the frame and increment the frame counter.
-		pthread_mutex_lock(param->mutexCoreExecute);
+		pthread_mutex_lock(&param->mutexCoreExecute);
 		NDS_exec<false>();
-		pthread_mutex_unlock(param->mutexCoreExecute);
+		pthread_mutex_unlock(&param->mutexCoreExecute);
 		
 		// Check if an internal execution error occurred that halted the emulation.
 		if (!execute)
@@ -700,15 +812,17 @@ static void* RunCoreThread(void *arg)
 			param->frameCount++;
 		}
 		
+		pthread_mutex_lock(&param->mutexOutputList);
 		for(CocoaDSOutput *cdsOutput in cdsOutputList)
 		{
-			if (param->framesToSkip > 0 && [cdsOutput isMemberOfClass:[CocoaDSDisplay class]])
+			if (param->framesToSkip > 0 && [cdsOutput isKindOfClass:[CocoaDSDisplay class]])
 			{
 				continue;
 			}
 			
 			[cdsOutput doCoreEmuFrame];
 		}
+		pthread_mutex_unlock(&param->mutexOutputList);
 		
 		// Determine the number of frames to skip based on how much time "debt"
 		// we owe on timeBudget.
