@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2012-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define STRING_DESMUME_WEBSITE						"http://www.desmume.org"
+#define STRING_DESMUME_WEBSITE						"http://desmume.org"
+#define STRING_DESMUME_SHORT_DESCRIPTION			"Nintendo DS Emulator"
 #define STRING_DESMUME_FORUM_SITE					"http://forums.desmume.org/index.php"
-#define STRING_DESMUME_BUG_SITE						"http://sourceforge.net/tracker/?group_id=164579&atid=832291"
+#define STRING_DESMUME_TECH_SUPPORT_SITE			"http://forums.desmume.org/viewforum.php?id=4"
+#define STRING_DESMUME_BUG_REPORT_SITE				"http://sourceforge.net/tracker/?group_id=164579&atid=832291"
 
 // User Interface Localized Strings
 #define NSSTRING_TITLE_OPEN_ROM_PANEL				NSLocalizedString(@"Open ROM", nil)
@@ -46,9 +48,18 @@
 #define NSSTRING_TITLE_ENABLE_HUD					NSLocalizedString(@"Enable HUD", nil)
 #define NSSTRING_TITLE_HIDE_STATUS_BAR				NSLocalizedString(@"Hide Status Bar", nil)
 #define NSSTRING_TITLE_SHOW_STATUS_BAR				NSLocalizedString(@"Show Status Bar", nil)
+#define NSSTRING_TITLE_HIDE_TOOLBAR					NSLocalizedString(@"Hide Toolbar", nil)
+#define NSSTRING_TITLE_SHOW_TOOLBAR					NSLocalizedString(@"Show Toolbar", nil)
 #define NSSTRING_TITLE_SPEED_1X						NSLocalizedString(@"Speed 1x", nil)
 #define NSSTRING_TITLE_SPEED_2X						NSLocalizedString(@"Speed 2x", nil)
-#define NSSTRING_TITLE_SLOT_NUMBER					NSLocalizedString(@"Slot %d", nil)
+#define NSSTRING_TITLE_SLOT_NUMBER					NSLocalizedString(@"Slot %ld", nil)
+
+#define NSSTRING_TITLE_TECH_SUPPORT_WINDOW_TITLE	NSLocalizedString(@"Support Request Form", nil)
+#define NSSTRING_TITLE_BUG_REPORT_WINDOW_TITLE		NSLocalizedString(@"Bug Report Form", nil)
+#define NSSTRING_TITLE_GO_TECH_SUPPORT_WEBPAGE_TITLE	NSLocalizedString(@"Go to Tech Support Webpage", nil)
+#define NSSTRING_TITLE_GO_BUG_REPORT_WEBPAGE_TITLE	NSLocalizedString(@"Go to Bug Report Webpage", nil)
+#define NSSTRING_HELP_COPY_PASTE_TECH_SUPPORT		NSLocalizedString(@"Please copy-paste the above information into our tech support webpage. This will ensure the fastest response time from us.", nil)
+#define NSSTRING_HELP_COPY_PASTE_BUG_REPORT			NSLocalizedString(@"Please copy-paste the above information into our bug report webpage. This will ensure the fastest response time from us.", nil)
 
 #define NSSTRING_ALERT_CRITICAL_FILE_MISSING_PRI	NSLocalizedString(@"A critical file is missing. DeSmuME will now quit.", nil)
 #define NSSTRING_ALERT_CRITICAL_FILE_MISSING_SEC	NSLocalizedString(@"The file \"DefaultUserPrefs.plist\" is missing. Please reinstall DeSmuME.", nil)
@@ -86,10 +97,32 @@
 #define NSSTRING_STATUS_NO_ROM_LOADED				NSLocalizedString(@"No ROM loaded.", nil)
 #define NSSTRING_STATUS_SIZE_BYTES					NSLocalizedString(@"%i bytes", nil)
 
+#define NSSTRING_STATUS_EMULATION_NOT_RUNNING		NSLocalizedString(@"Emulation is not running.", nil)
+#define NSSTRING_STATUS_SLOT1_UNKNOWN_STATE			NSLocalizedString(@"Unknown state.", nil)
+#define NSSTRING_STATUS_SLOT1_NO_DEVICE				NSLocalizedString(@"No device inserted.", nil)
+#define NSSTRING_STATUS_SLOT1_RETAIL_INSERTED		NSLocalizedString(@"Retail cartridge inserted.", nil)
+#define NSSTRING_STATUS_SLOT1_RETAIL_NAND_INSERTED	NSLocalizedString(@"Retail cartridge (with NAND flash) inserted.", nil)
+#define NSSTRING_STATUS_SLOT1_R4_INSERTED			NSLocalizedString(@"R4 cartridge interface inserted.", nil)
+
 #define NSSTRING_DISPLAYMODE_MAIN					NSLocalizedString(@"Main", nil)
 #define NSSTRING_DISPLAYMODE_TOUCH					NSLocalizedString(@"Touch", nil)
 #define NSSTRING_DISPLAYMODE_COMBO					NSLocalizedString(@"Combo", nil)
 
+#define NSSTRING_INPUTPREF_NUM_INPUTS_MAPPED		NSLocalizedString(@"%ld Input Mapped", nil)
+#define NSSTRING_INPUTPREF_NUM_INPUTS_MAPPED_PLURAL	NSLocalizedString(@"%ld Inputs Mapped", nil)
+
+#define NSSTRING_INPUTPREF_NO_SAVED_PROFILES		NSLocalizedString(@"No saved profiles.", nil)
+
+#define NSSTRING_INPUTPREF_USE_DEVICE_COORDINATES	NSLocalizedString(@"Use Device Coordinates", nil)
+#define NSSTRING_INPUTPREF_MIC_NONE					NSLocalizedString(@"None", nil)
+#define NSSTRING_INPUTPREF_MIC_INTERNAL_NOISE		NSLocalizedString(@"Internal Noise Samples", nil)
+#define NSSTRING_INPUTPREF_MIC_AUDIO_FILE_NONE_SELECTED	NSLocalizedString(@"No audio file selected.", nil)
+#define NSSTRING_INPUTPREF_MIC_WHITE_NOISE			NSLocalizedString(@"White Noise", nil)
+#define NSSTRING_INPUTPREF_MIC_SINE_WAVE			NSLocalizedString(@"%1.1f Hz Sine Wave", nil)
+#define NSSTRING_INPUTPREF_SPEED_SCALAR				NSLocalizedString(@"%1.2fx Speed", nil)
+#define NSSTRING_INPUTPREF_GPU_STATE_ALL_MAIN		NSLocalizedString(@"Main GPU - All Layers", nil)
+#define NSSTRING_INPUTPREF_GPU_STATE_ALL_SUB		NSLocalizedString(@"Sub GPU - All Layers", nil)
+
 // LEGACY STRINGS
 #define NSSTRING_TITLE_OPEN_ROM_PANEL_LEGACY		NSLocalizedString(@"Open ROM...", nil)
 #define NSSTRING_TITLE_OPEN_STATE_FILE_PANEL_LEGACY	NSLocalizedString(@"Load State From...", nil)
@@ -141,6 +174,9 @@
 #define GPU_DISPLAY_COLOR_DEPTH						sizeof(UInt16)
 #define GPU_SCREEN_SIZE_BYTES						(GPU_DISPLAY_WIDTH * GPU_DISPLAY_HEIGHT * GPU_DISPLAY_COLOR_DEPTH) // The numbers are: 256px width, 192px height, 16bit color depth
 
+#define DS_DISPLAY_VERTICAL_GAP_TO_HEIGHT_RATIO		(21.0/46.0) // Based on the official DS specification: 21mm/46mm
+#define DS_DISPLAY_GAP								(GPU_DISPLAY_HEIGHT * DS_DISPLAY_VERTICAL_GAP_TO_HEIGHT_RATIO)
+
 #define WINDOW_STATUS_BAR_HEIGHT					24		// Height of an emulation window status bar in pixels.
 
 #define SPEED_SCALAR_QUARTER						0.25	// Speed scalar for quarter execution speed.
@@ -150,7 +186,7 @@
 #define SPEED_SCALAR_DOUBLE							2.0		// Speed scalar for double execution speed.
 #define SPEED_SCALAR_MIN							0.005	// Lower limit for the speed multiplier.
 
-#define DS_FRAMES_PER_SECOND						59.73	// Number of DS frames per second.
+#define DS_FRAMES_PER_SECOND						59.8261	// Number of DS frames per second.
 #define DS_SECONDS_PER_FRAME						(1.0 / DS_FRAMES_PER_SECOND) // The length of time in seconds that, ideally, a frame should be processed within.
 
 #define FRAME_SKIP_AGGRESSIVENESS					10.0	// Must be a value between 0.0 (inclusive) and positive infinity.
@@ -179,10 +215,8 @@
 #define ROMINFO_GAME_CODE_LENGTH					4
 #define ROMINFO_GAME_BANNER_LENGTH					128
 
-#define MIC_NULL_SAMPLE_VALUE						0
 #define MIC_SAMPLE_RATE								16000
-#define MIC_MAX_BUFFER_SAMPLES						320
-#define MIC_BUFFER_SIZE								(sizeof(UInt8) * MIC_MAX_BUFFER_SAMPLES)
+#define MIC_MAX_BUFFER_SAMPLES						(MIC_SAMPLE_RATE / DS_FRAMES_PER_SECOND)
 
 #define COCOA_DIALOG_CANCEL							0
 #define COCOA_DIALOG_DEFAULT						1
@@ -225,7 +259,8 @@ enum
 	EMULATION_BOOT_FROM_FIRMWARE_BIT				= 6,
 	EMULATION_SLEEP_BIT								= 7,
 	EMULATION_CARD_EJECT_BIT						= 8,
-	EMULATION_DEBUG_CONSOLE_BIT						= 9
+	EMULATION_DEBUG_CONSOLE_BIT						= 9,
+	EMULATION_RIGOROUS_TIMING_BIT					= 10
 };
 
 enum
@@ -239,7 +274,14 @@ enum
 	EMULATION_BOOT_FROM_FIRMWARE_MASK				= 1 << EMULATION_BOOT_FROM_FIRMWARE_BIT,
 	EMULATION_SLEEP_MASK							= 1 << EMULATION_SLEEP_BIT,
 	EMULATION_CARD_EJECT_MASK						= 1 << EMULATION_CARD_EJECT_BIT,
-	EMULATION_DEBUG_CONSOLE_MASK					= 1 << EMULATION_DEBUG_CONSOLE_BIT
+	EMULATION_DEBUG_CONSOLE_MASK					= 1 << EMULATION_DEBUG_CONSOLE_BIT,
+	EMULATION_RIGOROUS_TIMING_MASK					= 1 << EMULATION_RIGOROUS_TIMING_BIT
+};
+
+enum
+{
+	CPU_EMULATION_ENGINE_INTERPRETER				= 0,
+	CPU_EMULATION_ENGINE_DYNAMIC_RECOMPILER			= 1
 };
 
 enum
@@ -337,10 +379,15 @@ enum
 	
 	// Video Messages
 	MESSAGE_RESIZE_VIEW,
+	MESSAGE_TRANSFORM_VIEW,
 	MESSAGE_REDRAW_VIEW,
 	MESSAGE_SET_GPU_STATE_FLAGS,
 	MESSAGE_CHANGE_DISPLAY_TYPE,
+	MESSAGE_CHANGE_DISPLAY_ORIENTATION,
+	MESSAGE_CHANGE_DISPLAY_ORDER,
+	MESSAGE_CHANGE_DISPLAY_GAP,
 	MESSAGE_CHANGE_BILINEAR_OUTPUT,
+	MESSAGE_CHANGE_VERTICAL_SYNC,
 	MESSAGE_CHANGE_VIDEO_FILTER,
 	MESSAGE_SET_RENDER3D_METHOD,
 	MESSAGE_SET_RENDER3D_HIGH_PRECISION_COLOR_INTERPOLATION,
@@ -350,6 +397,7 @@ enum
 	MESSAGE_SET_RENDER3D_DEPTH_COMPARISON_THRESHOLD,
 	MESSAGE_SET_RENDER3D_THREADS,
 	MESSAGE_SET_RENDER3D_LINE_HACK,
+	MESSAGE_SET_RENDER3D_MULTISAMPLE,
 	MESSAGE_SET_VIEW_TO_BLACK,
 	MESSAGE_SET_VIEW_TO_WHITE,
 	
@@ -374,6 +422,18 @@ enum
 	DS_DISPLAY_TYPE_COMBO
 };
 
+enum
+{
+	DS_DISPLAY_ORIENTATION_VERTICAL = 0,
+	DS_DISPLAY_ORIENTATION_HORIZONTAL
+};
+
+enum
+{
+	DS_DISPLAY_ORDER_MAIN_FIRST = 0,
+	DS_DISPLAY_ORDER_TOUCH_FIRST
+};
+
 /*
  DS GPU TYPES
  */
@@ -410,39 +470,8 @@ enum
 {
 	MICMODE_NONE = 0,
 	MICMODE_INTERNAL_NOISE,
-	MICMODE_SOUND_FILE,
+	MICMODE_AUDIO_FILE,
 	MICMODE_WHITE_NOISE,
-	MICMODE_PHYSICAL
+	MICMODE_PHYSICAL,
+	MICMODE_SINE_WAVE
 };
-
-/*
- CONTROL TAGS FOR PREFERENCE WINDOW INPUT CONFIG BUTTONS
- */
-enum
-{
-	PREF_INPUT_BUTTON_UP = 1000,
-	PREF_INPUT_BUTTON_DOWN,
-	PREF_INPUT_BUTTON_LEFT,
-	PREF_INPUT_BUTTON_RIGHT,
-	PREF_INPUT_BUTTON_A,
-	PREF_INPUT_BUTTON_B,
-	PREF_INPUT_BUTTON_X,
-	PREF_INPUT_BUTTON_Y,
-	PREF_INPUT_BUTTON_L,
-	PREF_INPUT_BUTTON_R,
-	PREF_INPUT_BUTTON_START,
-	PREF_INPUT_BUTTON_SELECT,
-	
-	PREF_INPUT_BUTTON_SIM_MIC = 2000,
-	
-	PREF_INPUT_BUTTON_LID = 3000,
-	PREF_INPUT_BUTTON_DEBUG,
-	PREF_INPUT_BUTTON_SPEED_HALF,
-	PREF_INPUT_BUTTON_SPEED_DOUBLE,
-	PREF_INPUT_BUTTON_TOGGLE_HUD,
-	PREF_INPUT_BUTTON_EXECUTE,
-	PREF_INPUT_BUTTON_PAUSE,
-	PREF_INPUT_BUTTON_RESET,
-	
-	PREF_INPUT_BUTTON_TOUCH = 4000
-};
\ No newline at end of file
