@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2011-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #import "cocoa_util.h"
 
 #include "../NDSSystem.h"
+#include "../MMU.h"
 #include "../mc.h"
 #undef BOOL
 
@@ -31,7 +32,7 @@
 @synthesize header;
 @synthesize bindings;
 @synthesize fileURL;
-@synthesize isDataLoaded;
+@dynamic isDataLoaded;
 @synthesize saveType;
 
 static NSMutableDictionary *saveTypeValues = nil;
@@ -87,7 +88,6 @@ static NSMutableDictionary *saveTypeValues = nil;
 	}
 	
 	fileURL = nil;
-	isDataLoaded = NO;
 	saveType = saveTypeID;
 	
 	xmlCurrentRom = nil;
@@ -104,10 +104,9 @@ static NSMutableDictionary *saveTypeValues = nil;
 
 - (void)dealloc
 {
-	if (isDataLoaded)
+	if ([self isDataLoaded])
 	{
 		NDS_FreeROM();
-		isDataLoaded = NO;
 	}
 	
 	[xmlElementStack release];
@@ -119,6 +118,11 @@ static NSMutableDictionary *saveTypeValues = nil;
 	[super dealloc];
 }
 
+- (BOOL) isDataLoaded
+{
+	return (MMU.CART_ROM != MMU.UNUSED_RAM);
+}
+
 - (void) initHeader
 {
 	const NDS_header *ndsRomHeader = NDS_getROMHeader();
@@ -131,12 +135,12 @@ static NSMutableDictionary *saveTypeValues = nil;
 	
 	if (&ndsRomBanner != NULL)
 	{
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[0]] forKey:@"bannerJapanese"];
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[1]] forKey:@"bannerEnglish"];
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[2]] forKey:@"bannerFrench"];
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[3]] forKey:@"bannerGerman"];
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[4]] forKey:@"bannerItalian"];
-		[self.header setValue:[self getRomBannerTitle:ndsRomBanner.titles[5]] forKey:@"bannerSpanish"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[0]] forKey:@"bannerJapanese"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[1]] forKey:@"bannerEnglish"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[2]] forKey:@"bannerFrench"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[3]] forKey:@"bannerGerman"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[4]] forKey:@"bannerItalian"];
+		[self.header setValue:[self banner:ndsRomBanner.titles[5]] forKey:@"bannerSpanish"];
 		
 		[self.bindings setValue:[self.header objectForKey:@"bannerJapanese"] forKey:@"bannerJapanese"];
 		[self.bindings setValue:[self.header objectForKey:@"bannerEnglish"] forKey:@"bannerEnglish"];
@@ -148,8 +152,8 @@ static NSMutableDictionary *saveTypeValues = nil;
 	
 	if (ndsRomHeader != NULL)
 	{
-		[self.header setValue:[self getRomTitle] forKey:@"gameTitle"];
-		[self.header setValue:[self getRomCode] forKey:@"gameCode"];
+		[self.header setValue:[self title] forKey:@"gameTitle"];
+		[self.header setValue:[self code] forKey:@"gameCode"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->makerCode] forKey:@"makerCode"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->cardSize] forKey:@"romSize"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->ARM9src] forKey:@"arm9BinaryOffset"];
@@ -196,11 +200,16 @@ static NSMutableDictionary *saveTypeValues = nil;
 		[self.bindings setObject:(NSImage *)[self.header objectForKey:@"iconImage"] forKey:@"iconImage"];
 	}
 	
-	[self.header setValue:[self getRomInternalName] forKey:@"romInternalName"];
-	[self.header setValue:[self getRomSerial] forKey:@"romSerial"];
+	[self.header setValue:[self internalName] forKey:@"romInternalName"];
+	[self.header setValue:[self serial] forKey:@"romSerial"];
 	
 	[self.bindings setValue:[self.header objectForKey:@"romInternalName"] forKey:@"romInternalName"];
 	[self.bindings setValue:[self.header objectForKey:@"romSerial"] forKey:@"romSerial"];
+	
+	NSString *romNameAndSerialInfoString = @"Name: ";
+	romNameAndSerialInfoString = [romNameAndSerialInfoString stringByAppendingString:[self.header objectForKey:@"romInternalName"]];
+	romNameAndSerialInfoString = [[romNameAndSerialInfoString stringByAppendingString:@"\nSerial: "] stringByAppendingString:[self.header objectForKey:@"romSerial"]];
+	[self.bindings setValue:romNameAndSerialInfoString forKey:@"romNameAndSerialInfo"];
 }
 
 - (BOOL) loadData:(NSURL *)theURL
@@ -217,7 +226,6 @@ static NSMutableDictionary *saveTypeValues = nil;
 	}
 	
 	fileURL = [theURL copy];
-	isDataLoaded = YES;
 	[self initHeader];
 	
 	NSString *advscDBPath = [[NSUserDefaults standardUserDefaults] stringForKey:@"Advanscene_DatabasePath"];
@@ -253,7 +261,7 @@ static NSMutableDictionary *saveTypeValues = nil;
 	[self release];
 }
 
-- (NSString *) getRomTitle
+- (NSString *) title
 {
 	NDS_header *ndsRomHeader = NDS_getROMHeader();
 	if (ndsRomHeader == nil)
@@ -264,7 +272,7 @@ static NSMutableDictionary *saveTypeValues = nil;
 	return [[[NSString alloc] initWithBytes:ndsRomHeader->gameTile length:ROMINFO_GAME_TITLE_LENGTH encoding:NSUTF8StringEncoding] autorelease];
 }
 
-- (NSString *) getRomCode
+- (NSString *) code
 {
 	NDS_header *ndsRomHeader = NDS_getROMHeader();
 	if (ndsRomHeader == nil)
@@ -275,19 +283,19 @@ static NSMutableDictionary *saveTypeValues = nil;
 	return [[[NSString alloc] initWithBytes:ndsRomHeader->gameCode length:ROMINFO_GAME_CODE_LENGTH encoding:NSUTF8StringEncoding] autorelease];
 }
 
-- (NSString *) getRomBannerTitle:(const UInt16 *)title
+- (NSString *) banner:(const UInt16 *)UTF16TextBuffer
 {
-	NSUInteger titleLength = ROMINFO_GAME_BANNER_LENGTH * sizeof(*title);
+	NSUInteger bannerLength = ROMINFO_GAME_BANNER_LENGTH * sizeof(*UTF16TextBuffer);
 	
-	return [[[NSString alloc] initWithBytes:title length:titleLength encoding:NSUTF16LittleEndianStringEncoding] autorelease];
+	return [[[NSString alloc] initWithBytes:UTF16TextBuffer length:bannerLength encoding:NSUTF16LittleEndianStringEncoding] autorelease];
 }
 
-- (NSString *) getRomInternalName
+- (NSString *) internalName
 {
 	return [NSString stringWithCString:gameInfo.ROMname encoding:NSUTF8StringEncoding];
 }
 
-- (NSString *) getRomSerial
+- (NSString *) serial
 {
 	return [NSString stringWithCString:gameInfo.ROMserial encoding:NSUTF8StringEncoding];
 }
@@ -477,9 +485,14 @@ static NSMutableDictionary *saveTypeValues = nil;
 {
 	NSImage *iconImage = [[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AppIcon_DeSmuME" ofType:@"icns"]] autorelease];
 	
+	NSString *romNameAndSerialInfoString = @"Name: ";
+	romNameAndSerialInfoString = [romNameAndSerialInfoString stringByAppendingString:NSSTRING_STATUS_NO_ROM_LOADED];
+	romNameAndSerialInfoString = [[romNameAndSerialInfoString stringByAppendingString:@"\nSerial: "] stringByAppendingString:NSSTRING_STATUS_NO_ROM_LOADED];
+	
 	return [NSMutableDictionary dictionaryWithObjectsAndKeys:
 			NSSTRING_STATUS_NO_ROM_LOADED, @"romInternalName",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"romSerial",
+			romNameAndSerialInfoString, @"romNameAndSerialInfo",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"bannerJapanese",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"bannerEnglish",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"bannerFrench",
@@ -561,14 +574,14 @@ void RomIconToRGBA8888(uint32_t *bitmapData)
 	iconClutPtr = (uint16_t *)&MMU.CART_ROM[iconOffset + 0x220] + 1;
 	iconPixPtr = (uint32_t *)&MMU.CART_ROM[iconOffset + 0x20];
 	
-	// Setup the 4-bit color CLUT.
+	// Setup the 4-bit CLUT.
 	//
-	// The actual color values are stored with the ROM icon data in RGBA5551 format.
+	// The actual color values are stored with the ROM icon data in RGB555 format.
 	// We convert these color values and store them in the CLUT as RGBA8888 values.
 	//
 	// The first entry always represents the alpha, so we can just ignore it.
 	clut[0] = 0x00000000;
-	RGBA5551ToRGBA8888Buffer(iconClutPtr, &clut[1], 15);
+	RGB555ToRGBA8888Buffer(iconClutPtr, &clut[1], 15);
 	
 	// Load the image from the icon pixel data.
 	//
