@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -19,31 +19,34 @@
 #define _RINGBUFFER_
 
 #include <stdlib.h>
-#include <string.h>
-#include <libkern/OSAtomic.h>
+#include <stdint.h>
 
 
 class RingBuffer
 {
 private:
 	uint8_t *_buffer;
-	uint8_t *_bufferEnd;
 	size_t _bufferSize;
-	size_t _numElements;
+	
+	size_t _elementCapacity;
 	size_t _elementSize;
-	int32_t _bufferFillSize;
+	int32_t _elementFillCount; // need to use int32_t for OSAtomicAdd32Barrier()
 	size_t _readPosition;
 	size_t _writePosition;
 	
 public:
-	RingBuffer(size_t numberElements, size_t newBufferElementSize);
+	RingBuffer(const size_t numberElements, const size_t newBufferElementSize);
 	~RingBuffer();
 	
 	void clear();
-	size_t read(void *__restrict__ destBuffer, size_t requestedNumberBytes);
-	size_t write(const void *__restrict__ srcBuffer, size_t requestedNumberBytes);
-	size_t getAvailableElements();
-	size_t getElementSize();
+	size_t read(void *__restrict__ destBuffer, size_t requestedNumberElements);
+	size_t write(const void *__restrict__ srcBuffer, size_t requestedNumberElements);
+	size_t drop(size_t requestedNumberElements);
+	size_t getAvailableElements() const;
+	size_t getElementCapacity() const;
+	size_t getElementSize() const;
+	bool isEmpty() const;
+	bool isFull() const;
 };
 
 #endif
