@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2011-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,38 +17,42 @@
 */
 
 #import "appDelegate.h"
-#import "emuWindowDelegate.h"
+#import "DisplayWindowController.h"
+#import "EmuControllerDelegate.h"
+#import "FileMigrationDelegate.h"
 #import "preferencesWindowDelegate.h"
+#import "troubleshootingWindowDelegate.h"
 #import "cheatWindowDelegate.h"
-#import "displayView.h"
 #import "inputPrefsView.h"
 
 #import "cocoa_core.h"
 #import "cocoa_file.h"
 #import "cocoa_firmware.h"
 #import "cocoa_globals.h"
-#import "cocoa_hid.h"
 #import "cocoa_input.h"
-#import "cocoa_mic.h"
+#import "cocoa_rom.h"
+#import "cocoa_util.h"
 
 
 @implementation AppDelegate
 
 @dynamic dummyObject;
-@synthesize mainWindow;
 @synthesize prefWindow;
+@synthesize troubleshootingWindow;
 @synthesize cheatListWindow;
-@synthesize migrationWindow;
 @synthesize prefGeneralView;
 @synthesize mLoadStateSlot;
 @synthesize mSaveStateSlot;
 @synthesize inputPrefsView;
-@synthesize fileMigrationList;
 @synthesize aboutWindowController;
-@synthesize emuWindowController;
+@synthesize emuControlController;
+@synthesize cdsSoundController;
+@synthesize romInfoPanelController;
 @synthesize prefWindowController;
 @synthesize cdsCoreController;
 @synthesize cheatWindowController;
+@synthesize migrationDelegate;
+@synthesize inputManager;
 
 @synthesize boxGeneralInfo;
 @synthesize boxTitles;
@@ -56,14 +60,32 @@
 @synthesize boxFileSystem;
 @synthesize boxMisc;
 
-@synthesize hidManager;
-@synthesize migrationFilesPresent;
+@synthesize isAppRunningOnIntel;
+
+
+- (id)init
+{
+	self = [super init];
+	if(self == nil)
+	{
+		return nil;
+	}
+	
+	// Determine if we're running on Intel or PPC.
+#if defined(__i386__) || defined(__x86_64__)
+	isAppRunningOnIntel = YES;
+#else
+	isAppRunningOnIntel = NO;
+#endif
+	
+	return self;
+}
 
 - (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
 {
 	BOOL result = NO;
 	NSURL *fileURL = [NSURL fileURLWithPath:filename];
-	EmuWindowDelegate *mainWindowDelegate = [mainWindow delegate];
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
 	
 	if (cdsCore == nil)
@@ -74,8 +96,8 @@
 	NSString *fileKind = [CocoaDSFile fileKindByURL:fileURL];
 	if ([fileKind isEqualToString:@"ROM"])
 	{
-		result = [mainWindowDelegate handleLoadRom:fileURL];
-		if ([mainWindowDelegate isShowingSaveStateSheet] || [mainWindowDelegate isShowingFileMigrationSheet])
+		result = [emuControl handleLoadRom:fileURL];
+		if ([emuControl isShowingSaveStateDialog] || [emuControl isShowingFileMigrationDialog])
 		{
 			// Just reply YES if a sheet is showing, even if the ROM hasn't actually been loaded yet.
 			// This will prevent the error dialog from showing, which is the intended behavior in
@@ -89,9 +111,9 @@
 
 - (void)applicationWillFinishLaunching:(NSNotification *)notification
 {
-	EmuWindowDelegate *mainWindowDelegate = [mainWindow delegate];
-	PreferencesWindowDelegate *prefWindowDelegate = [prefWindow delegate];
-	CheatWindowDelegate *cheatWindowDelegate = [cheatListWindow delegate];
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
+	PreferencesWindowDelegate *prefWindowDelegate = (PreferencesWindowDelegate *)[prefWindow delegate];
+	CheatWindowDelegate *cheatWindowDelegate = (CheatWindowDelegate *)[cheatListWindow delegate];
 	
 	// Create the needed directories in Application Support if they haven't already
 	// been created.
@@ -104,13 +126,25 @@
 	[CocoaDSFile setupAllFilePaths];
 	
 	// Setup the About window.
+	NSString *descriptionStr = [[[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"] stringByAppendingString:@" "] stringByAppendingString:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"]];
+	descriptionStr = [[descriptionStr stringByAppendingString:@"\n"] stringByAppendingString:@STRING_DESMUME_SHORT_DESCRIPTION];
+	descriptionStr = [[descriptionStr stringByAppendingString:@"\n"] stringByAppendingString:@STRING_DESMUME_WEBSITE];
+	
+	NSString *buildInfoStr = @"Build Info:";
+	buildInfoStr = [[buildInfoStr stringByAppendingString:[CocoaDSUtil appInternalVersionString]] stringByAppendingString:[CocoaDSUtil appCompilerDetailString]];
+	buildInfoStr = [[buildInfoStr stringByAppendingString:@"\nBuild Date: "] stringByAppendingString:@__DATE__];
+	buildInfoStr = [[buildInfoStr stringByAppendingString:@"\nOperating System: "] stringByAppendingString:[CocoaDSUtil operatingSystemString]];
+	buildInfoStr = [[buildInfoStr stringByAppendingString:@"\nModel Identifier: "] stringByAppendingString:[CocoaDSUtil modelIdentifierString]];
+	
+	NSFont *aboutTextFilesFont = [NSFont fontWithName:@"Monaco" size:10];
 	NSMutableDictionary *aboutWindowProperties = [NSMutableDictionary dictionaryWithObjectsAndKeys:
 												  [[NSBundle mainBundle] pathForResource:@FILENAME_README ofType:@""], @"readMePath",
 												  [[NSBundle mainBundle] pathForResource:@FILENAME_COPYING ofType:@""], @"licensePath",
 												  [[NSBundle mainBundle] pathForResource:@FILENAME_AUTHORS ofType:@""], @"authorsPath",
 												  [[NSBundle mainBundle] pathForResource:@FILENAME_CHANGELOG ofType:@""], @"changeLogPath",
-												  [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"], @"versionString",
-												  @__DATE__, @"dateString",
+												  descriptionStr, @"descriptionString",
+												  buildInfoStr, @"buildInfoString",
+												  aboutTextFilesFont, @"aboutTextFilesFont",
 												  nil];
 	
 	[aboutWindowController setContent:aboutWindowProperties];
@@ -120,7 +154,7 @@
 	if (prefsDict == nil)
 	{
 		[[NSAlert alertWithMessageText:NSSTRING_ALERT_CRITICAL_FILE_MISSING_PRI defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:NSSTRING_ALERT_CRITICAL_FILE_MISSING_SEC] runModal];
-		[NSApp terminate: nil];
+		[NSApp terminate:nil];
 		return;
 	}
 	
@@ -130,11 +164,6 @@
 	// colors manually here because you can't change them in Interface Builder. Boo!!!
 	[self setRomInfoPanelBoxTitleColors];
 	
-	// Set up all the object controllers according to our default windows.
-	[emuWindowController setContent:[mainWindowDelegate bindings]];
-	[prefWindowController setContent:[prefWindowDelegate bindings]];
-	[cheatWindowController setContent:[cheatWindowDelegate bindings]];
-	
 	// Set the preferences window to the general view by default.
 	[prefWindowDelegate switchContentView:prefGeneralView];
 	
@@ -142,37 +171,17 @@
 	// Builder because we're assuming an arbitrary number of slot items.
 	[self setupSlotMenuItems];
 	
-	// Setup the HID Input manager.
-	[self setHidManager:[[[CocoaHIDManager alloc] init] autorelease]];
-	
-	// Set the display view delegate.
-	DisplayViewDelegate *newDispViewDelegate = [[[DisplayViewDelegate alloc] init] autorelease];
-	[newDispViewDelegate setView:[mainWindowDelegate displayView]];
-	[mainWindowDelegate setDispViewDelegate:newDispViewDelegate];
-	[[mainWindowDelegate displayView] setDispViewDelegate:newDispViewDelegate];
-	
 	// Init the DS emulation core.
 	CocoaDSCore *newCore = [[[CocoaDSCore alloc] init] autorelease];
-	[cdsCoreController setContent:newCore];
 	
-	// Init the DS controller and microphone.
-	CocoaDSMic *newMic = [[[CocoaDSMic alloc] init] autorelease];
-	CocoaDSController *newController = [[[CocoaDSController alloc] initWithMic:newMic] autorelease];
+	// Init the DS controller.
+	CocoaDSController *newController = [[[CocoaDSController alloc] init] autorelease];
 	[newCore setCdsController:newController];
-	[inputPrefsView setCdsController:newController];
-	[newDispViewDelegate setCdsController:newController];
-	
-	// Init the DS displays.
-	CocoaDSDisplay *newComboDisplay = [[[CocoaDSDisplay alloc] init] autorelease];
-	[newComboDisplay setDelegate:newDispViewDelegate];
-	[newCore addOutput:newComboDisplay];
-	NSPort *guiPort = [NSPort port];
-	[[NSRunLoop currentRunLoop] addPort:guiPort forMode:NSDefaultRunLoopMode];
 	
 	// Init the DS speakers.
 	CocoaDSSpeaker *newSpeaker = [[[CocoaDSSpeaker alloc] init] autorelease];
 	[newCore addOutput:newSpeaker];
-	[mainWindowDelegate setCdsSpeaker:newSpeaker];
+	[emuControl setCdsSpeaker:newSpeaker];
 	
 	// Start the core thread.
 	[NSThread detachNewThreadSelector:@selector(runThread:) toTarget:newCore withObject:nil];
@@ -184,31 +193,55 @@
 	}
 	
 	// Start up the threads for our outputs.
-	[NSThread detachNewThreadSelector:@selector(runThread:) toTarget:newComboDisplay withObject:nil];
 	[NSThread detachNewThreadSelector:@selector(runThread:) toTarget:newSpeaker withObject:nil];
 	
-	// Wait until the GPU and SPU are finished starting up.
-	while ([newComboDisplay thread] == nil || [newSpeaker thread] == nil)
+	// Wait until the SPU is finished starting up.
+	while ([newSpeaker thread] == nil)
 	{
 		[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.01]];
 	}
 	
+	// Set up all the object controllers.
+	[cdsCoreController setContent:newCore];
+	[romInfoPanelController setContent:[CocoaDSRom romNotLoadedBindings]];
+	[prefWindowController setContent:[prefWindowDelegate bindings]];
+	[cheatWindowController setContent:[cheatWindowDelegate bindings]];
+	
 	// Setup the applications settings from the user defaults file.
 	[self setupUserDefaults];
 	
-	// Setup the window display view.
-	[[mainWindowDelegate displayView] setNextResponder:mainWindow];
+	[inputPrefsView initSettingsSheets];
+	[inputPrefsView populateInputProfileMenu];
+	[[inputPrefsView inputProfileMenu] selectItemAtIndex:0];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
 {
-	EmuWindowDelegate *mainWindowDelegate = [mainWindow delegate];
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
+	
+	// Determine if the app was run for the first time.
+	NSMutableDictionary *appFirstTimeRunDict = [[NSMutableDictionary alloc] initWithDictionary:[[NSUserDefaults standardUserDefaults] dictionaryForKey:@"General_AppFirstTimeRun"]];
+	NSString *bundleVersionString = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
+	
+	BOOL isFirstTimeRun = NO;
+	NSNumber *isFirstTimeRunNumber = (NSNumber *)[appFirstTimeRunDict valueForKey:bundleVersionString];
+	if (isFirstTimeRunNumber == nil)
+	{
+		isFirstTimeRunNumber = [NSNumber numberWithBool:isFirstTimeRun];
+	}
+	
+	isFirstTimeRun = [isFirstTimeRunNumber boolValue];
+	
+	if (appFirstTimeRunDict == nil)
+	{
+		appFirstTimeRunDict = [[NSMutableDictionary alloc] initWithObjectsAndKeys:isFirstTimeRunNumber, bundleVersionString, nil];
+	}
 	
 	// Load a new ROM on launch per user preferences.
-	BOOL loadROMOnLaunch = [[NSUserDefaults standardUserDefaults] boolForKey:@"General_AutoloadROMOnLaunch"];
-	if (loadROMOnLaunch && ![mainWindowDelegate isRomLoaded])
+	const BOOL loadROMOnLaunch = [[NSUserDefaults standardUserDefaults] boolForKey:@"General_AutoloadROMOnLaunch"];
+	if (loadROMOnLaunch && [emuControl currentRom] == nil)
 	{
-		NSInteger autoloadRomOption = [[NSUserDefaults standardUserDefaults] integerForKey:@"General_AutoloadROMOption"];
+		const NSInteger autoloadRomOption = [[NSUserDefaults standardUserDefaults] integerForKey:@"General_AutoloadROMOption"];
 		NSURL *autoloadRomURL = nil;
 		
 		if (autoloadRomOption == ROMAUTOLOADOPTION_LOAD_LAST)
@@ -226,36 +259,42 @@
 		
 		if (autoloadRomURL != nil)
 		{
-			[mainWindowDelegate handleLoadRom:autoloadRomURL];
+			[emuControl handleLoadRom:autoloadRomURL];
 		}
 	}
 	
-	// Make the display view black.
-	[[mainWindowDelegate dispViewDelegate] setViewToBlack];
-	
 	//Bring the application to the front
 	[NSApp activateIgnoringOtherApps:TRUE];
-	[mainWindow makeKeyAndOrderFront:nil];
-	[mainWindow makeMainWindow];
+	[emuControl newDisplayWindow:nil];
 	
 	// Present the file migration window to the user (if they haven't disabled it).
-	[self setMigrationFilesPresent:NO];
-	if (![[NSUserDefaults standardUserDefaults] boolForKey:@"General_DoNotAskMigrate"])
+	if (![[NSUserDefaults standardUserDefaults] boolForKey:@"General_DoNotAskMigrate"] || !isFirstTimeRun)
 	{
-		[self showMigrationWindow:nil];
+		[migrationDelegate updateFileList];
+		if ([migrationDelegate filesPresent])
+		{
+			[[migrationDelegate window] center];
+			[[migrationDelegate window] makeKeyAndOrderFront:nil];
+		}
 	}
+	
+	// Set that the app has run for the first time.
+	isFirstTimeRun = YES;
+	[appFirstTimeRunDict setValue:[NSNumber numberWithBool:isFirstTimeRun] forKey:bundleVersionString];
+	[[NSUserDefaults standardUserDefaults] setObject:appFirstTimeRunDict forKey:@"General_AppFirstTimeRun"];
+	[appFirstTimeRunDict release];
 }
 
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
 {
-	EmuWindowDelegate *mainWindowDelegate = [mainWindow delegate];
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	
 	// If a file needs to be saved, give the user a chance to save it
 	// before quitting.
-	BOOL didRomClose = [mainWindowDelegate handleUnloadRom:REASONFORCLOSE_TERMINATE romToLoad:nil];
+	const BOOL didRomClose = [emuControl handleUnloadRom:REASONFORCLOSE_TERMINATE romToLoad:nil];
 	if (!didRomClose)
 	{
-		if ([mainWindowDelegate isShowingSaveStateSheet])
+		if ([emuControl isShowingSaveStateDialog])
 		{
 			return NSTerminateLater;
 		}
@@ -268,14 +307,14 @@
 
 - (void)applicationWillTerminate:(NSNotification *)notification
 {
-	[cdsCoreController setContent:nil];
-}
-
-- (IBAction) showSupportFolderInFinder:(id)sender
-{
-	NSURL *folderURL = [CocoaDSFile userAppSupportBaseURL];
+	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
 	
-	[[NSWorkspace sharedWorkspace] openFile:[folderURL path] withApplication:@"Finder"];
+	// Save some settings to user defaults before app termination
+	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isSpeedLimitEnabled] forKey:@"CoreControl_EnableSpeedLimit"];
+	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isFrameSkipEnabled] forKey:@"CoreControl_EnableAutoFrameSkip"];
+	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isCheatingEnabled] forKey:@"CoreControl_EnableCheats"];
+	
+	[cdsCoreController setContent:nil];
 }
 
 - (IBAction) launchWebsite:(id)sender
@@ -288,18 +327,30 @@
 	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@STRING_DESMUME_FORUM_SITE]];
 }
 
+- (IBAction) supportRequest:(id)sender
+{
+	TroubleshootingWindowDelegate *troubleshootingWindowDelegate = [troubleshootingWindow delegate];
+	
+	[troubleshootingWindowDelegate switchViewByID:TROUBLESHOOTING_TECH_SUPPORT_VIEW_ID];
+	[troubleshootingWindow setTitle:NSSTRING_TITLE_TECH_SUPPORT_WINDOW_TITLE];
+	[troubleshootingWindow makeKeyAndOrderFront:sender];
+}
+
 - (IBAction) bugReport:(id)sender
 {
-	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@STRING_DESMUME_BUG_SITE]];
+	TroubleshootingWindowDelegate *troubleshootingWindowDelegate = [troubleshootingWindow delegate];
+	
+	[troubleshootingWindowDelegate switchViewByID:TROUBLESHOOTING_BUG_REPORT_VIEW_ID];
+	[troubleshootingWindow setTitle:NSSTRING_TITLE_BUG_REPORT_WINDOW_TITLE];
+	[troubleshootingWindow makeKeyAndOrderFront:sender];
 }
 
 - (void) setupSlotMenuItems
 {
-	NSInteger i;
 	NSMenuItem *loadItem = nil;
 	NSMenuItem *saveItem = nil;
 	
-	for(i = 0; i < MAX_SAVESTATE_SLOTS; i++)
+	for(NSInteger i = 0; i < MAX_SAVESTATE_SLOTS; i++)
 	{
 		loadItem = [self addSlotMenuItem:mLoadStateSlot slotNumber:(NSUInteger)(i + 1)];
 		[loadItem setKeyEquivalentModifierMask:0];
@@ -315,6 +366,7 @@
 
 - (NSMenuItem *) addSlotMenuItem:(NSMenu *)menu slotNumber:(NSUInteger)slotNumber
 {
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	NSUInteger slotNumberKey = slotNumber;
 	
 	if (slotNumber == 10)
@@ -322,18 +374,18 @@
 		slotNumberKey = 0;
 	}
 	
-	NSMenuItem *mItem = [menu addItemWithTitle:[NSString stringWithFormat:NSSTRING_TITLE_SLOT_NUMBER, slotNumber]
+	NSMenuItem *mItem = [menu addItemWithTitle:[NSString stringWithFormat:NSSTRING_TITLE_SLOT_NUMBER, (unsigned long)slotNumber]
 										action:nil
-								 keyEquivalent:[NSString stringWithFormat:@"%d", slotNumberKey]];
+								 keyEquivalent:[NSString stringWithFormat:@"%ld", (unsigned long)slotNumberKey]];
 	
-	[mItem setTarget:[mainWindow delegate]];
+	[mItem setTarget:emuControl];
 	
 	return mItem;
 }
 
 - (void) setupUserDefaults
 {
-	EmuWindowDelegate *mainWindowDelegate = [mainWindow delegate];
+	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	PreferencesWindowDelegate *prefWindowDelegate = [prefWindow delegate];
 	NSMutableDictionary *prefBindings = [prefWindowDelegate bindings];
 	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
@@ -346,6 +398,11 @@
 		emuFlags |= EMULATION_ADVANCED_BUS_LEVEL_TIMING_MASK;
 	}
 	
+	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"Emulation_RigorousTiming"])
+	{
+		emuFlags |= EMULATION_RIGOROUS_TIMING_MASK;
+	}
+	
 	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"Emulation_UseExternalBIOSImages"])
 	{
 		emuFlags |= EMULATION_USE_EXTERNAL_BIOS_MASK;
@@ -383,6 +440,26 @@
 	
 	[cdsCore setEmulationFlags:emuFlags];
 	
+	// If we're not running on Intel, force the CPU emulation engine to use the interpreter engine.
+	if (!isAppRunningOnIntel)
+	{
+		[[NSUserDefaults standardUserDefaults] setInteger:CPU_EMULATION_ENGINE_INTERPRETER forKey:@"Emulation_CPUEmulationEngine"];
+	}
+	
+	// Set the CPU emulation engine per user preferences.
+	[cdsCore setCpuEmulationEngine:[[NSUserDefaults standardUserDefaults] integerForKey:@"Emulation_CPUEmulationEngine"]];
+	[cdsCore setMaxJITBlockSize:[[NSUserDefaults standardUserDefaults] integerForKey:@"Emulation_MaxJITBlockSize"]];
+	
+	// Set the SLOT-1 device settings per user preferences.
+	NSString *slot1R4Path = (NSString *)[[NSUserDefaults standardUserDefaults] objectForKey:@"EmulationSlot1_R4StoragePath"];
+	[cdsCore setSlot1DeviceType:[[NSUserDefaults standardUserDefaults] integerForKey:@"EmulationSlot1_DeviceType"]];
+	[cdsCore setSlot1R4URL:(slot1R4Path != nil) ? [NSURL fileURLWithPath:slot1R4Path] : nil];
+	
+	// Set the miscellaneous emulations settings per user preferences.
+	[cdsCore setIsSpeedLimitEnabled:[[NSUserDefaults standardUserDefaults] boolForKey:@"CoreControl_EnableSpeedLimit"]];
+	[cdsCore setIsFrameSkipEnabled:[[NSUserDefaults standardUserDefaults] boolForKey:@"CoreControl_EnableAutoFrameSkip"]];
+	[cdsCore setIsCheatingEnabled:[[NSUserDefaults standardUserDefaults] boolForKey:@"CoreControl_EnableCheats"]];
+	
 	// Set up the firmware per user preferences.
 	NSMutableDictionary *newFWDict = [NSMutableDictionary dictionaryWithObjectsAndKeys:
 									  [[NSUserDefaults standardUserDefaults] objectForKey:@"FirmwareConfig_Nickname"], @"Nickname",
@@ -393,8 +470,8 @@
 									  nil];
 	
 	CocoaDSFirmware *newFirmware = [[[CocoaDSFirmware alloc] initWithDictionary:newFWDict] autorelease];
-	[cdsCore setCdsFirmware:newFirmware];
 	[newFirmware update];
+	[emuControl setCdsFirmware:newFirmware];
 	
 	// Setup the ARM7 BIOS, ARM9 BIOS, and firmware image paths per user preferences.
 	NSString *arm7BiosImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"BIOS_ARM7ImagePath"];
@@ -445,9 +522,6 @@
 		[prefBindings setValue:[cheatDatabasePath lastPathComponent] forKey:@"R4CheatDatabaseName"];
 	}
 	
-	// Set the sound input mode per user preferences.
-	[[cdsCore cdsController] setSoundInputMode:[[NSUserDefaults standardUserDefaults] integerForKey:@"Input_AudioInputMode"]];
-	
 	// Update the SPU Sync controls in the Preferences window.
 	if ([[NSUserDefaults standardUserDefaults] integerForKey:@"SPU_SyncMode"] == SPU_SYNC_MODE_DUAL_SYNC_ASYNC)
 	{
@@ -469,97 +543,26 @@
 		[prefBindings setValue:nil forKey:@"AutoloadRomName"];
 	}
 	
-	// Update all of the input display fields.
-	[self updateInputDisplayFields];
-	
 	// Set the menu for the display rotation.
-	double displayRotation = (double)[[NSUserDefaults standardUserDefaults] floatForKey:@"DisplayView_Rotation"];
+	const double displayRotation = (double)[[NSUserDefaults standardUserDefaults] floatForKey:@"DisplayView_Rotation"];
 	[prefWindowDelegate updateDisplayRotationMenu:displayRotation];
 	
 	// Set the default sound volume per user preferences.
 	[prefWindowDelegate updateVolumeIcon:nil];
 	
-	[mainWindowDelegate setupUserDefaults];
-}
-
-- (void) updateInputDisplayFields
-{
-	PreferencesWindowDelegate *prefWindowDelegate = [prefWindow delegate];
-	NSMutableDictionary *prefBindings = [prefWindowDelegate bindings];
-	
-	if ([[NSUserDefaults standardUserDefaults] dictionaryForKey:@"Input_ControllerMappings"] == nil)
+	// Set up the user's default input settings.
+	NSDictionary *userMappings = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"Input_ControllerMappings"];
+	if (userMappings == nil)
 	{
-		return;
+		NSDictionary *defaultKeyMappingsDict = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"DefaultKeyMappings" ofType:@"plist"]];
+		NSArray *internalDefaultProfilesList = (NSArray *)[defaultKeyMappingsDict valueForKey:@"DefaultInputProfiles"];
+		userMappings = [(NSDictionary *)[internalDefaultProfilesList objectAtIndex:0] valueForKey:@"Mappings"];
 	}
 	
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Up"] forKey:@"Input_Up"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Down"] forKey:@"Input_Down"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Left"] forKey:@"Input_Left"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Right"] forKey:@"Input_Right"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"A"] forKey:@"Input_A"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"B"] forKey:@"Input_B"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"X"] forKey:@"Input_X"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Y"] forKey:@"Input_Y"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"L"] forKey:@"Input_L"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"R"] forKey:@"Input_R"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Start"] forKey:@"Input_Start"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Select"] forKey:@"Input_Select"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Microphone"] forKey:@"Input_Microphone"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Lid"] forKey:@"Input_Lid"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Debug"] forKey:@"Input_Debug"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Speed Half"] forKey:@"Input_SpeedHalf"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Speed Double"] forKey:@"Input_SpeedDouble"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"HUD"] forKey:@"Input_HUD"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Execute"] forKey:@"Input_Execute"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Pause"] forKey:@"Input_Pause"];
-	[prefBindings setValue:[inputPrefsView parseMappingDisplayString:@"Reset"] forKey:@"Input_Reset"];
-}
-
-- (IBAction) showMigrationWindow:(id)sender
-{
-	NSMutableArray *fileList = [CocoaDSFile completeFileList];
-	if (fileList != nil)
-	{
-		if ([fileList count] == 0)
-		{
-			[self setMigrationFilesPresent:NO];
-		}
-		else
-		{
-			[self setMigrationFilesPresent:YES];
-		}
-		
-		if (sender == nil && ![self migrationFilesPresent])
-		{
-			return;
-		}
-	}
+	[inputManager setMappingsWithMappings:userMappings];
 	
-	[fileMigrationList setContent:fileList];
-	[migrationWindow center];
-	[migrationWindow makeKeyAndOrderFront:nil];
-}
-
-- (IBAction) handleMigrationWindow:(id)sender
-{
-	NSInteger option = [(NSControl *)sender tag];
-	NSMutableArray *fileList = [fileMigrationList content];
-	
-	switch (option)
-	{
-		case COCOA_DIALOG_DEFAULT:
-			[CocoaDSFile copyFileListToCurrent:fileList];
-			break;
-		
-		case COCOA_DIALOG_OPTION:
-			[CocoaDSFile moveFileListToCurrent:fileList];
-			break;
-		
-		default:
-			break;
-	}
-	
-	[[(NSControl *)sender window] close];
+	// Set up the rest of the emulation-related user defaults.
+	[emuControl setupUserDefaults];
 }
 
 - (void) setRomInfoPanelBoxTitleColors
