@@ -1,24 +1,20 @@
-/* joysdl.c - this file is part of DeSmuME
- *
- * Copyright (C) 2007 Pascal Giard
- *
- * Author: Pascal Giard <evilynux@gmail.com>
- *
- * This file is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This file is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+/*
+	Copyright (C) 2007 Pascal Giard
+	Copyright (C) 2007-2011 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "ctrlssdl.h"
 #include "saves.h"
@@ -428,7 +424,7 @@ process_ctrls_event( SDL_Event& event,
     switch (event.type)
     {
       case SDL_VIDEORESIZE:
-        cfg->resize_cb( event.resize.w, event.resize.h);
+        cfg->resize_cb( event.resize.w, event.resize.h, cfg->screen_texture);
         break;
 
       case SDL_ACTIVEEVENT:
@@ -436,7 +432,7 @@ process_ctrls_event( SDL_Event& event,
           if (event.active.gain) {
             cfg->focused = 1;
             SPU_Pause(0);
-            osd->addLine("Auto pause disabled\n");
+            osd->addLine("Auto pause disabled");
           } else {
             cfg->focused = 0;
             SPU_Pause(1);
@@ -470,18 +466,18 @@ process_ctrls_event( SDL_Event& event,
                 cfg->fake_mic = !cfg->fake_mic;
                 Mic_DoNoise(cfg->fake_mic);
                 if (cfg->fake_mic)
-                  osd->addLine("Fake mic enabled\n");
+                  osd->addLine("Fake mic enabled");
                 else
-                  osd->addLine("Fake mic disabled\n");
+                  osd->addLine("Fake mic disabled");
                 break;
 #endif
 
             case SDLK_o:
                 cfg->boost = !cfg->boost;
                 if (cfg->boost)
-                  osd->addLine("Boost mode enabled\n");
+                  osd->addLine("Boost mode enabled");
                 else
-                  osd->addLine("Boost mode disabled\n");
+                  osd->addLine("Boost mode disabled");
                 break;
 
             case SDLK_LSHIFT:
