@@ -1,28 +1,28 @@
-/* joysdl.h - this file is part of DeSmuME
- *
- * Copyright (C) 2007 Pascal Giard
- *
- * Author: Pascal Giard <evilynux@gmail.com>
- *
- * This file is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This file is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+/*
+	Copyright (C) 2007 Pascal Giard
+	Copyright (C) 2007-2011 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef CTRLSSDL_H
 #define CTRLSSDL_H
 
+#ifdef HAVE_GL_GL_H
+#include <GL/gl.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -90,7 +90,13 @@ struct ctrls_event_config {
   int sdl_quit;
   int boost;
   int fake_mic;
-  void (*resize_cb)(u16 width, u16 height);
+#ifdef HAVE_GL_GL_H
+  GLuint *screen_texture;
+  void (*resize_cb)(u16 width, u16 height, GLuint *screen_texture);
+#else
+  void *screen_texture;
+  void (*resize_cb)(u16 width, u16 height, void *screen_texture);
+#endif
 };
 
 void load_default_config(const u16 kbCfg[]);
