@@ -1,20 +1,18 @@
-/*	Copyright (C) 2009 DeSmuME Team
+/*
+	Copyright (C) 2009-2011 DeSmuME Team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _FIRMWARE_H_
@@ -24,6 +22,8 @@
 // the count of bytes copied from the firmware into memory 
 #define NDS_FW_USER_SETTINGS_MEM_BYTE_COUNT 0x70
 
+#define FW_CONFIG_FILE_EXT "dfc"
+
 class CFIRMWARE
 {
 private:
@@ -48,6 +48,8 @@ public:
 	
 	bool load();
 
+	static std::string GetExternalFilePath();
+
 	struct HEADER
 	{
 		u16	part3_rom_gui9_addr;		// 000h
