@@ -10,7 +10,9 @@ typedef void (*dTool_closeFn)();
 
 typedef struct
 {
-	const char name[64];
+	/* this should be the same name of the action in the gui xml */
+	const char shortname[16];
+	const char name[32];
 	dTool_openFn open;
 	dTool_updateFn update;
 	dTool_closeFn close;
