@@ -1,5 +1,6 @@
-/*  Copyright (C) 2006-2007 shash
-	Copyright (C) 2009-2010 DeSmuME team
+/*  
+	Copyright (C) 2006-2007 shash
+	Copyright (C) 2007-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -50,14 +51,11 @@ void	MatrixInit				(s32 *matrix);
 
 float	MatrixGetMultipliedIndex	(int index, float *matrix, float *rightMatrix);
 s32	MatrixGetMultipliedIndex	(int index, s32 *matrix, s32 *rightMatrix);
-void	MatrixSet				(float *matrix, int x, int y, float value);
-void	MatrixCopy				(float * matrixDST, const float * matrixSRC);
+void	MatrixSet				(s32 *matrix, int x, int y, s32 value);
 void	MatrixCopy				(s32 * matrixDST, const s32 * matrixSRC);
-int		MatrixCompare				(const float * matrixDST, const float * matrixSRC);
-void	MatrixIdentity			(float *matrix);
+int		MatrixCompare				(const s32 * matrixDST, const float * matrixSRC);
 void	MatrixIdentity			(s32 *matrix);
 
-void	MatrixTranspose				(float *matrix);
 void	MatrixStackInit				(MatrixStack *stack);
 void	MatrixStackSetMaxSize		(MatrixStack *stack, int size);
 void	MatrixStackPushMatrix		(MatrixStack *stack, const s32 *ptr);
@@ -113,6 +111,10 @@ FORCEINLINE s32 s32floor(float f)
 	return (s32)floorf(f);
 #endif
 }
+FORCEINLINE s32 s32floor(double d)
+{
+	return s32floor((float)d);
+}
 
 //switched SSE2 functions
 //-------------
