@@ -1,5 +1,6 @@
-/*  Copyright (C) 2006 Theo Berkau
-	Copyright (C) 2006-2010 DeSmuME team
+/*
+	Copyright (C) 2006 Theo Berkau
+	Copyright (C) 2006-2011 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -347,6 +348,9 @@
 #define eng_3D_CLIPMTX_RESULT  0x04000640
 #define eng_3D_VECMTX_RESULT   0x04000680
 
+//DSI
+#define REG_DSIMODE 0x04004000
+
 #define IPCFIFOCNT_SENDEMPTY 0x0001
 #define IPCFIFOCNT_SENDFULL 0x0002
 #define IPCFIFOCNT_SENDIRQEN 0x0004
@@ -358,6 +362,9 @@
 #define IPCFIFOCNT_FIFOENABLE 0x8000
 #define IPCFIFOCNT_WRITEABLE (IPCFIFOCNT_SENDIRQEN | IPCFIFOCNT_RECVIRQEN | IPCFIFOCNT_FIFOENABLE)
 
+#define IPCSYNC_IRQ_SEND 0x2000
+#define IPCSYNC_IRQ_RECV 0x4000
+
 #define IRQ_BIT_LCD_VBLANK 0
 #define IRQ_BIT_LCD_HBLANK 1
 #define IRQ_BIT_LCD_VMATCH 2
@@ -406,5 +413,19 @@
 #define IRQ_MASK_ARM7_SPI (1<<23)
 #define IRQ_MASK_ARM7_WIFI (1<<24)
 
+#define TSC_MEASURE_TEMP1    0
+#define TSC_MEASURE_Y        1
+#define TSC_MEASURE_BATTERY  2
+#define TSC_MEASURE_Z1       3
+#define TSC_MEASURE_Z2       4
+#define TSC_MEASURE_X        5
+#define TSC_MEASURE_AUX      6
+#define TSC_MEASURE_TEMP2    7
+
+#define EXMEMCNT_MASK_SLOT2_ARM7 (1<<7)
+#define EXMEMCNT_MASK_SLOT2_SRAM_TIME (3)
+#define EXMEMCNT_MASK_SLOT2_ROM_1ST_TIME (3<<2)
+#define EXMEMCNT_MASK_SLOT2_ROM_2ND_TIME (1<<4)
+#define EXMEMCNT_MASK_SLOT2_CLOCKRATE (3<<5)
 
 #endif
