@@ -2,7 +2,7 @@
 	Copyright (C) 2006 Normmatt
 	Copyright (C) 2006 Theo Berkau
 	Copyright (C) 2007 Pascal Giard
-	Copyright (C) 2008-2012 DeSmuME team
+	Copyright (C) 2008-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+//MUST SAVE ADDONS! OMG HOW DID WE FORGET THAT
+
 #ifdef HAVE_LIBZ
 #include <zlib.h>
 #endif
@@ -132,7 +134,7 @@ SFORMAT SF_ARM9[]={
 	{ "9LDT", 1, 1, &NDS_ARM9.LDTBit},
 	{ "9Wai", 4, 1, &NDS_ARM9.waitIRQ},
 	{ "9hef", 4, 1, &NDS_ARM9.halt_IE_and_IF },
-	{ "9iws", 1, 1, &NDS_ARM7.intrWaitARM_state },
+	{ "9iws", 1, 1, &NDS_ARM9.intrWaitARM_state },
 	{ 0 }
 };
 
@@ -214,6 +216,8 @@ SFORMAT SF_MMU[]={
 	{ "MASX", 1, 2,       &MMU.AUX_SPI_CNT},
 	{ "MASC", 1, 2,       &MMU.AUX_SPI_CMD},
 
+	{ "MWRA", 1, 2,       &MMU.WRAMCNT},
+
 	{ "MDV1", 4, 1,       &MMU.divRunning},
 	{ "MDV2", 8, 1,       &MMU.divResult},
 	{ "MDV3", 8, 1,       &MMU.divMod},
@@ -270,7 +274,7 @@ SFORMAT SF_MOVIE[]={
 
 static void mmu_savestate(EMUFILE* os)
 {
-	u32 version = 6;
+	u32 version = 8;
 	write32le(version,os);
 	
 	//version 2:
@@ -295,6 +299,10 @@ static void mmu_savestate(EMUFILE* os)
 
 	//version 6:
 	MMU_new.dsi_tsc.save_state(os);
+
+	//version 8:
+	os->write32le(MMU.fw.size);
+	os->fwrite(MMU.fw.data,MMU.fw.size);
 }
 
 // TODO: integrate the new wifi state variables once everything is settled
@@ -419,7 +427,7 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 
 	bool ok = MMU_new.backupDevice.load_state(is);
 
-	if(version < 3) return true;
+	if(version < 3) return ok;
 
 	ok &= MMU_new.gxstat.loadstate(is);
 	
@@ -434,7 +442,7 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 	ok &= MMU_timing.arm9codeCache.loadstate(is, version);
 	ok &= MMU_timing.arm9dataCache.loadstate(is, version);
 
-	if(version < 4) return true;
+	if(version < 4) return ok;
 
 	ok &= MMU_new.sqrt.loadstate(is,version);
 	ok &= MMU_new.div.loadstate(is,version);
@@ -453,50 +461,22 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 
 	MMU_new.dsi_tsc.load_state(is);
 
-	return ok;
-}
+	//version 6
+	if(version < 7)
+	{
+		//recover WRAMCNT from the stashed WRAMSTAT memory location
+		MMU.WRAMCNT = MMU.MMU_MEM[ARMCPU_ARM7][0x40][0x241];
+	}
 
-static void cp15_saveone(armcp15_t *cp15, EMUFILE* os)
-{
-	write32le(cp15->IDCode,os);
-	write32le(cp15->cacheType,os);
-    write32le(cp15->TCMSize,os);
-    write32le(cp15->ctrl,os);
-    write32le(cp15->DCConfig,os);
-    write32le(cp15->ICConfig,os);
-    write32le(cp15->writeBuffCtrl,os);
-    write32le(cp15->und,os);
-    write32le(cp15->DaccessPerm,os);
-    write32le(cp15->IaccessPerm,os);
-    write32le(cp15->protectBaseSize0,os);
-    write32le(cp15->protectBaseSize1,os);
-    write32le(cp15->protectBaseSize2,os);
-    write32le(cp15->protectBaseSize3,os);
-    write32le(cp15->protectBaseSize4,os);
-    write32le(cp15->protectBaseSize5,os);
-    write32le(cp15->protectBaseSize6,os);
-    write32le(cp15->protectBaseSize7,os);
-    write32le(cp15->cacheOp,os);
-    write32le(cp15->DcacheLock,os);
-    write32le(cp15->IcacheLock,os);
-    write32le(cp15->ITCMRegion,os);
-    write32le(cp15->DTCMRegion,os);
-    write32le(cp15->processID,os);
-    write32le(cp15->RAM_TAG,os);
-    write32le(cp15->testState,os);
-    write32le(cp15->cacheDbg,os);
-    for(int i=0;i<8;i++) write32le(cp15->regionWriteMask_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionWriteMask_SYS[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionReadMask_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionReadMask_SYS[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionExecuteMask_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionExecuteMask_SYS[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionWriteSet_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionWriteSet_SYS[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionReadSet_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionReadSet_SYS[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionExecuteSet_USR[i],os);
-    for(int i=0;i<8;i++) write32le(cp15->regionExecuteSet_SYS[i],os);
+	if(version<8) return ok;
+
+	//version 8:
+	delete[] MMU.fw.data;
+	MMU.fw.size = is->read32le();
+	MMU.fw.data = new u8[size];
+	is->fread(MMU.fw.data,MMU.fw.size);
+
+	return ok;
 }
 
 static void cp15_savestate(EMUFILE* os)
@@ -504,56 +484,11 @@ static void cp15_savestate(EMUFILE* os)
 	//version
 	write32le(1,os);
 
-	cp15_saveone((armcp15_t *)NDS_ARM9.coproc[15],os);
+	cp15.saveone(os);
 	//ARM7 not have coprocessor
 	//cp15_saveone((armcp15_t *)NDS_ARM7.coproc[15],os);
 }
 
-static bool cp15_loadone(armcp15_t *cp15, EMUFILE* is)
-{
-	if(!read32le(&cp15->IDCode,is)) return false;
-	if(!read32le(&cp15->cacheType,is)) return false;
-    if(!read32le(&cp15->TCMSize,is)) return false;
-    if(!read32le(&cp15->ctrl,is)) return false;
-    if(!read32le(&cp15->DCConfig,is)) return false;
-    if(!read32le(&cp15->ICConfig,is)) return false;
-    if(!read32le(&cp15->writeBuffCtrl,is)) return false;
-    if(!read32le(&cp15->und,is)) return false;
-    if(!read32le(&cp15->DaccessPerm,is)) return false;
-    if(!read32le(&cp15->IaccessPerm,is)) return false;
-    if(!read32le(&cp15->protectBaseSize0,is)) return false;
-    if(!read32le(&cp15->protectBaseSize1,is)) return false;
-    if(!read32le(&cp15->protectBaseSize2,is)) return false;
-    if(!read32le(&cp15->protectBaseSize3,is)) return false;
-    if(!read32le(&cp15->protectBaseSize4,is)) return false;
-    if(!read32le(&cp15->protectBaseSize5,is)) return false;
-    if(!read32le(&cp15->protectBaseSize6,is)) return false;
-    if(!read32le(&cp15->protectBaseSize7,is)) return false;
-    if(!read32le(&cp15->cacheOp,is)) return false;
-    if(!read32le(&cp15->DcacheLock,is)) return false;
-    if(!read32le(&cp15->IcacheLock,is)) return false;
-    if(!read32le(&cp15->ITCMRegion,is)) return false;
-    if(!read32le(&cp15->DTCMRegion,is)) return false;
-    if(!read32le(&cp15->processID,is)) return false;
-    if(!read32le(&cp15->RAM_TAG,is)) return false;
-    if(!read32le(&cp15->testState,is)) return false;
-    if(!read32le(&cp15->cacheDbg,is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionWriteMask_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionWriteMask_SYS[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionReadMask_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionReadMask_SYS[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionExecuteMask_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionExecuteMask_SYS[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionWriteSet_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionWriteSet_SYS[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionReadSet_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionReadSet_SYS[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionExecuteSet_USR[i],is)) return false;
-    for(int i=0;i<8;i++) if(!read32le(&cp15->regionExecuteSet_SYS[i],is)) return false;
-
-    return true;
-}
-
 static bool cp15_loadstate(EMUFILE* is, int size)
 {
 	//read version
@@ -561,14 +496,14 @@ static bool cp15_loadstate(EMUFILE* is, int size)
 	if(read32le(&version,is) != 1) return false;
 	if(version > 1) return false;
 
-	if(!cp15_loadone((armcp15_t *)NDS_ARM9.coproc[15],is)) return false;
+	if(!cp15.loadone(is)) return false;
 	
 	if(version == 0)
 	{
 		//ARM7 not have coprocessor
 		u8 *tmp_buf = new u8 [sizeof(armcp15_t)];
 		if (!tmp_buf) return false;
-		if(!cp15_loadone((armcp15_t *)tmp_buf,is)) return false;
+		if(!cp15.loadone(is)) return false;
 		delete [] tmp_buf;
 		tmp_buf = NULL;
 	}
@@ -902,6 +837,9 @@ static void writechunks(EMUFILE* os);
 
 bool savestate_save(EMUFILE* outstream, int compressionLevel)
 {
+#ifdef HAVE_JIT 
+	arm_jit_sync();
+#endif
 	#ifndef HAVE_LIBZ
 	compressionLevel = Z_NO_COMPRESSION;
 	#endif
