@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2010-2011 DeSmuME team
+	Copyright (C) 2010-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "types.h"
 #include "slot1.h"
 
+#include "NDSSystem.h"
 #include "emufile.h"
 #include "utils/vfat.h"
 
@@ -39,7 +40,7 @@ SLOT1INTERFACE slot1List[NDS_SLOT1_COUNT] = {
 };
 
 SLOT1INTERFACE slot1_device = slot1Retail; //default for frontends that dont even configure this
-u8 slot1_device_type = NDS_SLOT1_RETAIL;
+NDS_SLOT1_TYPE slot1_device_type = NDS_SLOT1_RETAIL;
 
 static void scanDir()
 {
@@ -84,6 +85,7 @@ void slot1Reset()
 
 BOOL slot1Change(NDS_SLOT1_TYPE changeToType)
 {
+	if(changeToType == slot1_device_type) return FALSE; //nothing to do
 	if (changeToType > NDS_SLOT1_COUNT || changeToType < 0) return FALSE;
 	slot1_device.close();
 	slot1_device_type = changeToType;
@@ -91,6 +93,8 @@ BOOL slot1Change(NDS_SLOT1_TYPE changeToType)
 	if (changeToType == NDS_SLOT1_R4)
 		scanDir();
 	printf("Slot 1: %s\n", slot1_device.name);
+	printf("sending eject signal to SLOT-1\n");
+	NDS_TriggerCardEjectIRQ();
 	return slot1_device.init();
 }
 
@@ -108,4 +112,9 @@ std::string slot1GetFatDir()
 EMUFILE* slot1GetFatImage()
 {
 	return fatImage;
+}
+
+NDS_SLOT1_TYPE slot1GetCurrentType()
+{
+	return slot1_device_type;
 }
\ No newline at end of file
