@@ -1,3 +1,22 @@
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2006-2007 shash
+	Copyright (C) 2008-2011 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 #include <string.h>
 #include <algorithm>
 #include <assert.h>
@@ -302,6 +321,8 @@ public:
 			//the texture matches params, but isnt suspected invalid. accept it.
 			if (!curr->suspectedInvalid) return curr;
 
+			//we suspect the texture may be invalid. we need to do a byte-for-byte comparison to re-establish that it is valid:
+
 			//when the palettes dont match:
 			//note that we are considering 4x4 textures to have a palette size of 0.
 			//they really have a potentially HUGE palette, too big for us to handle like a normal palette,
@@ -321,6 +342,7 @@ public:
 			//REMINDER to make it primary/newest when we have smarter code
 			//list_remove(curr);
 			//list_push_front(curr);
+			curr->suspectedInvalid = false;
 			return curr;
 
 		REJECT:
