@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2005 Guillaume Duhamel
-	Copyright (C) 2008-2011 DeSmuME team
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 	#ifdef DEVELOPER
 		#define HAVE_LUA
 	#endif
+	#define HAVE_JIT
 #endif
 
 #ifdef __GNUC__
@@ -95,16 +96,18 @@
 //use this for example when you want a byte value to be better-aligned
 #define FAST_ALIGN DS_ALIGN(4)
 
-#ifndef FASTCALL
 #ifdef __MINGW32__
 #define FASTCALL __attribute__((fastcall))
+#define ASMJIT_CALL_CONV kX86FuncConvGccFastCall
 #elif defined (__i386__) && !defined(__clang__)
 #define FASTCALL __attribute__((regparm(3)))
+#define ASMJIT_CALL_CONV kX86FuncConvGccRegParm3
 #elif defined(_MSC_VER) || defined(__INTEL_COMPILER)
 #define FASTCALL
+#define ASMJIT_CALL_CONV kX86FuncConvDefault
 #else
 #define FASTCALL
-#endif
+#define ASMJIT_CALL_CONV kX86FuncConvDefault
 #endif
 
 #ifdef _MSC_VER
@@ -131,6 +134,14 @@
 #endif
 #endif
 
+#ifndef NOINLINE
+#ifdef __GNUC__
+#define NOINLINE __attribute__((noinline))
+#else
+#define NOINLINE
+#endif
+#endif
+
 #if defined(__LP64__)
 typedef unsigned char u8;
 typedef unsigned short u16;
