@@ -21,6 +21,6 @@
 #ifndef _DECRYPT_H_
 #define _DECRYPT_H_
 
-void DecryptSecureArea(u8 *romdata, long romlen);
+bool DecryptSecureArea(u8 *romdata, long romlen);
 
 #endif
